      real*8, allocatable :: U_KNOT(:), V_KNOT(:),W_KNOT(:)
      real*8, allocatable :: UN_KNOT(:), VN_KNOT(:),WN_KNOT(:)      
      integer NSD,P,Q,R,MCP,NCP,OCP
      real*8  ksu,ksv,ksw
      integer nue,nev,new
      real*8  asu,asv,asw,x,y
      integer meshf,knotfile
      integer flg,i,j,k

      character*10 inp
c
c...  Read in preliminary information
c
      meshf = 11
      open (meshf, file='mesh.dat', status='old')
      read (meshf,*)  NSD           ! number of spatial dimensions
      read (meshf,*)  P, Q, R       ! degree of curves in u, v, and w direction
      read (meshf,*)  MCP, NCP, OCP ! number of control points in each direction

c
c...  Allocate arrays for knot vectors 
c
      allocate(U_KNOT(MCP+P+1))    
      allocate(V_KNOT(NCP+Q+1))
      allocate(W_KNOT(OCP+R+1))

c
c...  Read knot vectors and control net
c
      do i = 1,MCP+P+1
         if (i < MCP+P+1) then
            read (meshf,"(F13.9)", ADVANCE = "NO") U_KNOT(i) ! read U_KNOT
         else
            read (meshf,"(F13.9)", ADVANCE = "YES") U_KNOT(i) 
         endif
      enddo

      do i = 1,NCP+Q+1
         if (i < NCP+Q+1) then
            read (meshf,"(F13.9)", ADVANCE = "NO") V_KNOT(i) ! read V_KNOT
         else
            read (meshf,"(F13.9)", ADVANCE = "YES") V_KNOT(i) 
         endif
      enddo

      do i = 1,OCP+R+1
         if (i < OCP+R+1) then
            read (meshf,"(F13.9)", ADVANCE = "NO") W_KNOT(i) ! read W_KNOT
         else
            read (meshf,"(F13.9)", ADVANCE = "YES") W_KNOT(i) 
         endif
      enddo

c
c...  Get size of knot span
c
      ksu=U_KNOT(MCP+R+1) - U_KNOT(1)
      ksv=V_KNOT(NCP+R+1) - V_KNOT(1)
      ksw=W_KNOT(OCP+R+1) - W_KNOT(1)

      call IArgC(i)
      if (i.ne.6) then  
        call getarg(1,inp); read(inp,*) neu
        call getarg(2,inp); read(inp,*) nev
        call getarg(3,inp); read(inp,*) new
     
        call getarg(4,inp); read(inp,*) asu
        call getarg(5,inp); read(inp,*) asv
        call getarg(6,inp); read(inp,*) asw
      else
c
c...  Get number of elements
c
        write(*,*)"Elements in u direction? "
        read(*,*) neu
        write(*,*)"Elements in v direction? " 
        read(*,*) nev
        write(*,*)"Elements in w direction? "
        read(*,*) new
c
c...  Get stretching
c
        write(*,*)"Stretch in u direction? 0-no: "
        read(*,*) asu
        write(*,*)"Stretch in v direction? 0-no: " 
        read(*,*) asv      
        write(*,*)"Stretch in w direction? 0-no: " 
        read(*,*) asw    
      endif
c
c...  Allocate arrays for knot vectors 
c
      allocate(UN_KNOT(neu+1))
      allocate(VN_KNOT(nev+1))
      allocate(WN_KNOT(new+1))

      k=0
      do i = 0,neu
         x=real(i)/real(neu)
         call stretch(y,x,asu)
         x = y*ksu
         flg=0
         do j = 1,MCP+P+1 
           if (x == U_KNOT(j)) then
             flg=1
           endif
         enddo
         if (flg == 0) then
            k=k+1
            UN_KNOT(k)=x
         endif
      enddo
      neu=k
      
      k=0
      do i = 0,nev
         x=real(i)/real(nev)
         call stretch(y,x,asv)
         x = y*ksv
         flg=0
         do j = 1,NCP+Q+1
           if (x == V_KNOT(j)) then
             flg=1
           endif
         enddo
         if (flg == 0) then
            k=k+1
            VN_KNOT(k)=x
         endif
      enddo
      nev=k

      k=0
      do i = 0,new
         x=real(i)/real(new)
         call stretch(y,x,asw)
         x = y*ksw
         flg=0
         do j = 1,OCP+R+1
           if (x == W_KNOT(j)) then
             flg=1
           endif
         enddo
         if (flg == 0) then
            k=k+1
            WN_KNOT(k)=x
         endif
      enddo
      new=k
c
c...  Read in preliminary information
c
      knotfile = 12
      open  (knotfile, file='knot.dat', status='unknown')
      if (neu == 0) then
        write (knotfile,"(I5)", ADVANCE = "NO")  0
      else
        write (knotfile,"(I5)", ADVANCE = "NO")  1
      endif
      if (nev == 0) then
        write (knotfile,"(I5)", ADVANCE = "NO")  0
      else
        write (knotfile,"(I5)", ADVANCE = "NO")  1
      endif
      
      if (new == 0) then
        write (knotfile,"(I5)", ADVANCE = "YES")  0
      else
        write (knotfile,"(I5)", ADVANCE = "YES")  1
      endif  
      
      write (knotfile,"(I5)", ADVANCE = "NO")  neu
      write (knotfile,"(I5)", ADVANCE = "NO")  nev
      write (knotfile,"(I5)", ADVANCE = "YES") new
        
c
c...  Write knot vectors and control net
c
      do i = 1,neu
         if (i < neu) then
            write (knotfile,"(F13.9)", ADVANCE = "NO")  UN_KNOT(i) 
         else
            write (knotfile,"(F13.9)", ADVANCE = "YES") UN_KNOT(i)  
         endif
      enddo

      do i = 1,nev
         if (i < nev) then
            write (knotfile,"(F13.9)", ADVANCE = "NO")  VN_KNOT(i) 
         else
            write (knotfile,"(F13.9)", ADVANCE = "YES") VN_KNOT(i)     
         endif
      enddo

      do i = 1,new
         if (i < new) then
            write (knotfile,"(F13.9)", ADVANCE = "NO")  WN_KNOT(i) 
         else
            write (knotfile,"(F13.9)", ADVANCE = "YES") WN_KNOT(i)    
         endif
      enddo

      end


      subroutine stretch(y,x,a)

      real*8 y,x,a
      
      if (a <= 0) then
        y = x
      else
        y = ((tanh(a*(2*x-1.0))/tanh(a)) + 1.0)*0.5
      endif
      end subroutine
