      program gen_knot    

      implicit none

      real*8, allocatable :: U_KNOT(:), V_KNOT(:),W_KNOT(:),NU_KNOT(:)
      real*8, allocatable :: UN_KNOT(:), VN_KNOT(:),WN_KNOT(:)      
      integer NSD,P,Q,R,MCP,NCP,OCP
      integer elems(3)
      real*8  lu,lv,lw,li
      integer, allocatable :: elemu(:),elemv(:),elemw(:)
      real*8,  allocatable :: erru(:), errv(:), errw(:)
      real*8,  allocatable :: hu(:), hv(:), hw(:)
      integer ku,kv,kw,k,ia
      real*8  asu,asv,asw
      real*8  maxh, minh,h
      
      real*8  du, ratio(3)
      
      integer meshf,knotfile
      integer i,j
      character*10 inp

c
c...  Read in preliminary information
c
      open (meshf, file='mesh.dat', status='old')
      
      read (meshf,*)  NSD           ! number of spatial dimensions
      read (meshf,*)  P, Q, R       ! degree of curves in u, v, and w direction
      read (meshf,*)  MCP, NCP, OCP ! number of control points in each direction

c
c...  Allocate arrays for knot vectors 
c
      allocate(U_KNOT(MCP+P+1))    
      allocate(V_KNOT(NCP+Q+1))
      allocate(W_KNOT(OCP+R+1))

c
c...  Read knot vectors and control net

      do i = 1,MCP+P+1
         if (i < MCP+P+1) then
            read (meshf,"(F13.9)", ADVANCE = "NO" ) U_KNOT(i) ! read U_KNOT
         else
            read (meshf,"(F13.9)", ADVANCE = "YES") U_KNOT(i) 
         endif
      enddo
      write (*,*) "Finished reading U knot"
      do i = 1,NCP+Q+1
         if (i < NCP+Q+1) then
            read (meshf,"(F13.9)", ADVANCE = "NO" ) V_KNOT(i) ! read V_KNOT
         else
            read (meshf,"(F13.9)", ADVANCE = "YES") V_KNOT(i) 
         endif
      enddo
      write (*,*) "Finished reading V knot"
      do i = 1,OCP+R+1
         if (i < OCP+R+1) then
            read (meshf,"(F13.9)", ADVANCE = "NO" ) W_KNOT(i) ! read W_KNOT
         else
            read (meshf,"(F13.9)", ADVANCE = "YES") W_KNOT(i) 
         endif
      enddo
      write (*,*) "Finished reading W knot"

c
c...  Get size of knot span
c
      i = iargc()
    
      if (i.eq.3) then 
    
        call getarg(1,inp); read(inp,*) elems(1)
        call getarg(2,inp); read(inp,*) elems(2)
        call getarg(3,inp); read(inp,*) elems(3)
	ia = 0 
	
	asu = -1d0
        asv = -1d0
        asw = -1d0

      else if (i.eq.6) then 
 
        call getarg(1,inp); read(inp,*) elems(1)
        call getarg(2,inp); read(inp,*) elems(2)
        call getarg(3,inp); read(inp,*) elems(3)

        call getarg(4,inp); read(inp,*) asu
        call getarg(5,inp); read(inp,*) asv
        call getarg(6,inp); read(inp,*) asw
          
	ia = 0
		
      else
c
c...  Get number of elements
c
        write(*,*)"Elements in u direction? "
        read(*,*) elems(1)
        write(*,*)"Elements in v direction? " 
        read(*,*) elems(2)
        write(*,*)"Elements in w direction? "
        read(*,*) elems(3)

c
c...  Get stretching
c
        write(*,*)"Stretch in u direction? 0-no: "
        read(*,*) asu
        write(*,*)"Stretch in v direction? 0-no: " 
        read(*,*) asv      
        write(*,*)"Stretch in w direction? 0-no: " 
        read(*,*) asw   
        ia = 1
      endif

c      write(*,*) elems
      
      lu = U_KNOT(MCP+R+1) - U_KNOT(1)
      lv = V_KNOT(NCP+R+1) - V_KNOT(1)
      lw = W_KNOT(OCP+R+1) - W_KNOT(1)

      
c 
c...  Get element partition
c
      allocate(elemu(MCP-P))    
      allocate(elemv(NCP-Q))
      allocate(elemw(OCP-R))      
      
      call partition(U_KNOT,P,MCP, elems(1), elemu, ratio(1),ia)  
      call partition(V_KNOT,Q,NCP, elems(2), elemv, ratio(2),ia)  	   
      call partition(W_KNOT,R,OCP, elems(3), elemw, ratio(3),ia)  	 
      
c      write(*,*) "-------------------------------"
c      write(*,*) elemu
c      write(*,*) elemv
c      write(*,*) elemw
c      write(*,*) "-------------------------------"
c      write(*,2000) ratio


c 
c...  Allocate arrays for knot vectors 
c

      ku = elems(1) - (MCP - P)
      kv = elems(2) - (NCP - Q)
      kw = elems(3) - (OCP - R)

      allocate(UN_KNOT(elems(1)))
      allocate(VN_KNOT(elems(2)))
      allocate(WN_KNOT(elems(3)))


      k=0        
      do i = 1, MCP-P     
        minh = 999d9
	maxh = -99d9     			
        do j = 1,elemu(i)-1
	   k = k + 1
	   call stretch(du,real(j)/real(elemu(i)),asu)	
	   UN_KNOT(k) = U_KNOT(i+P) + 
     &	               (U_KNOT(i+P+1) - U_KNOT(i+P))*du
     
           if (j.ge.2) then
	     h = UN_KNOT(k)-UN_KNOT(k-1)
           else
	     h = UN_KNOT(k)-U_KNOT(i+P)
	   endif
	   	   	     
	    minh=min(minh,h)
	    maxh=max(maxh,h)
        enddo	
        !write(*,*) "X ", i ,minh/maxh
      enddo 
      ku = k
      
      if (k.ne.ku) write(*,*)"ERROR u",k,ku

      k=0        
      do i = 1, NCP-Q   
        minh = 999d9
	maxh = -99d9          
        do j = 1,elemv(i)-1
	   k = k + 1	   
	   call stretch(du,real(j)/real(elemv(i)),asv)
	   VN_KNOT(k) = V_KNOT(i+Q) + 
     &	               (V_KNOT(i+Q+1) - V_KNOT(i+Q))*du
           if (j.ge.2) then
	     h = VN_KNOT(k)-VN_KNOT(k-1)
           else
	     h = VN_KNOT(k)-V_KNOT(i+Q)
	   endif
	   	   	     
	   minh=min(minh,h)
	   maxh=max(maxh,h)
        enddo	
        !write(*,*) "Y ", i ,minh/maxh	
      enddo 
       kv = k
      if (k.ne.kv) write(*,*)"ERROR v",k,kv

                  
      k=0        
      do i = 1, OCP-R 
        minh = 999d9
	maxh = -99d9            
        do j = 1,elemw(i)-1
	   k = k + 1
	   call stretch(du,real(j)/real(elemw(i)),asw)
	   WN_KNOT(k) = W_KNOT(i+R) + 
     &	               (W_KNOT(i+R+1) - W_KNOT(i+R))*du
           if (j.ge.2) then
	     h = WN_KNOT(k)-WN_KNOT(k-1)
           else
	     h = WN_KNOT(k)-W_KNOT(i+R)
	   endif
	   	   	     
	    minh=min(minh,h)
	    maxh=max(maxh,h)
        enddo	
        !write(*,*) "Z ", i ,minh/maxh
      enddo 
        kw= k
      if (k.ne.kw) write(*,*)"ERROR w",k,kw


      
c
c...  Read in preliminary information
c
      knotfile = 12
      open  (knotfile, file='knot.dat', status='unknown')
      if (ku == 0) then
        write (knotfile,"(I5)", ADVANCE = "NO")  0
      else
        write (knotfile,"(I5)", ADVANCE = "NO")  1
      endif
      
      if (kv == 0) then
        write (knotfile,"(I5)", ADVANCE = "NO")  0
      else
        write (knotfile,"(I5)", ADVANCE = "NO")  1
      endif
      
      if (kw == 0) then
        write (knotfile,"(I5)", ADVANCE = "YES") 0
      else
        write (knotfile,"(I5)", ADVANCE = "YES") 1
      endif  
      
      write (knotfile,"(I5)", ADVANCE = "NO")  ku 
      write (knotfile,"(I5)", ADVANCE = "NO")  kv
      write (knotfile,"(I5)", ADVANCE = "YES") kw 
        
c
c...  Write knot vectors and control net
c
      do i = 1,ku
         if (i < ku) then
            write (knotfile,"(F13.9)", ADVANCE = "NO")  UN_KNOT(i) 
         else
            write (knotfile,"(F13.9)", ADVANCE = "YES") UN_KNOT(i)  
         endif
      enddo

      do i = 1,kv
         if (i < kv) then
            write (knotfile,"(F13.9)", ADVANCE = "NO")  VN_KNOT(i) 
         else
            write (knotfile,"(F13.9)", ADVANCE = "YES") VN_KNOT(i)     
         endif
      enddo

      do i = 1,kw
         if (i < kw) then
            write (knotfile,"(F13.9)", ADVANCE = "NO")  WN_KNOT(i) 
         else
            write (knotfile,"(F13.9)", ADVANCE = "YES") WN_KNOT(i)    
         endif
      enddo

2000  format(F8.4,F8.4,F8.4)
      end
      


