!------------------------------------------------------------------
! subroutine to find the connectivities and generate the lwork
! files for parallel computing
!
! Variables:
!   GNNum : stores global nodal numbering of all local nodes
!           '-1' was assigned for empty entities..
!   RelNd : relative nodal information
!           e.g. RelNd(4,3,2)=1 means the global nodal numbering
!                of proc4 and proc3's second overlapping node is 1
!   CoMap: connectivity map
!           e.g CoMap(4,3) = 1 and CoMap(3,4) = 0 means
!               proc4 is master(receive) and proc3 is slave(send)
!               CoMap(:,:) = -1 means no link between two proc's
!   NOvNd : Numbers of Overlapping Nodes
!   RelPr : Relative processor No.
!
! NOTE: NP-1 is the maximum possible tasks
!------------------------------------------------------------------
subroutine gen_lwork(eps,NP, LNNod, xg, IPER, ConNP, &
                     MCP, NCP, OCP)

  integer,      intent(in) :: NP, LNNod(:), IPER(:,:), ConNP(:,:), &
                            MCP(:), NCP(:), OCP(:)
  real(kind=8), intent(in) :: xg(:,:,:)

  integer :: i, j, k, l, m, n, ii, jj, mm, nn
  integer :: i1, i2, i3, j1, j2, j3  
  integer :: flag, ier, UMF(NP), Pflag(NP)
  integer :: RelNd(NP,NP,MAXVAL(LNNod))
  integer :: NOvNd(NP,NP), CoMap(NP,NP)
  integer :: Ntask, p, datalen, msgnum, msglen, RelPr
  integer :: temp1((NP-1),NP), temp2(NP,NP)
  integer :: msgtag((NP-1),NP)

  real(kind=8) :: eps, xgold(NP,MAXVAL(LNNod),3), dist

  integer, allocatable :: task(:,:)
  character(len=30)    :: fname
  character(len=10)    :: cname
  
  eps = 1.0d-3*eps

  RelNd = -1
  CoMap = -1
  NOvNd = 0
  msgtag = 100

  xgold = 10000.0d0



  ! loop through all the processors, starting from NP
  do m = NP, 2, -1   ! master m
    write(*,*) '!== working on master processor:', m, ' ==!'
    l = 0
    do n = m-1, 1, -1   ! slave n
      
      ! check if master and slave meets
      if (ConNP(m,n) == 1) then

        write(*,*) '    !-- slave processor:', n, ' --!'
        ! compare slave with master
        k = 0
	
        ! loop through local nodes
        i = 0
        do i3 = 1, OCP(n)
        do i2 = 1, NCP(n)
        do i1 = 1, MCP(n)
        i = i + 1
        ! for every slave nodes on the boundary
        ! check with master nodes
        if (i3==1 .or. i3==OCP(n) .or. &
            i2==1 .or. i2==NCP(n) .or. &
            i1==1 .or. i1==MCP(n)) then

!        do i = 1, LNNod(n)     ! for every slave nodes

          j = 0
          do j3 = 1, OCP(m)
          do j2 = 1, NCP(m)
          do j1 = 1, MCP(m)
          j = j + 1
          ! ckeck with master nodes only on the boundary
          if (j3==1 .or. j3==OCP(m) .or. &
              j2==1 .or. j2==NCP(m) .or. &
              j1==1 .or. j1==MCP(m)) then

!          do j = 1, LNNod(m)   ! check with master nodes



	

            ! check 'proc m-1' with 'proc m' for overlapping nodes
            ! check if their distance < eps, IPER(i) == i (master)
            dist = (xg(i,1,n)-xg(j,1,m))**2 + (xg(i,2,n)-xg(j,2,m))**2 + &
                   (xg(i,3,n)-xg(j,3,m))**2
            dist = sqrt(dist)
	    
            if (dist < eps .and. IPER(i,n) == i .and. IPER(j,m) == j) then
	      
!          if (GNNum(i,n) == GNNum(j,m)) then

              ! check if the overlapping node belongs to
              ! higher order processors
              flag = 0
              loop1: do ii = m+1, NP  ! m means higher order master
	         !    ALSO check with self!!
                ! check only if two processors meet
			
                if ((ConNP(ii,n) == 1)) then
                  do jj = 1, LNNod(ii)

                    dist = (xg(i,1,n)-xgold(ii,jj,1))**2 + &
                           (xg(i,2,n)-xgold(ii,jj,2))**2 + &
                           (xg(i,3,n)-xgold(ii,jj,3))**2
                    dist = sqrt(dist)
                    if (dist < eps) then
                      flag = 1
                      exit loop1
                    end if
                  end do
                end if
              end do loop1            

              ! store the overlapping node
              if (flag == 0) then
                k = k + 1
!              RelNd(m,n,k) = GNNum(i,n)
!              RelNd(n,m,k) = GNNum(i,n)
                RelNd(m,n,k) = j
                RelNd(n,m,k) = i
              
                l = l + 1
                xgold(m,l,:) = xg(i,:,n) ! store the overlap location

                ! 'proc m' is master and 'proc n' is slave
                CoMap(m,n) = 1
                CoMap(n,m) = 0
              end if
            end if
           end if
          end do
          end do
          end do
        end if
        end do
        end do
        end do

!          end do
!        end do


        ! stores the number of overlapping nodes
        NOvNd(m,n) = k
        NOvNd(n,m) = k

      end if
    end do
  end do
 
  ! find the task info and total number of tasks
  ! always check through the largest to smallest proc ID
  temp2 = CoMap    ! for tmp CoMap
  temp1 = 0        ! for tmp task
  k     = 0
  do m = NP, 2, -1       ! master
    do n = m-1, 1, -1    ! slave
      Pflag = 1
      if (temp2(m,n) == 1) then
        Pflag(m) = 0
        Pflag(n) = 0
        k = k + 1

        temp1(k,m) = n   ! setup the task link
        temp1(k,n) = m
        temp2(m,n) = -1  ! if the link has been visited
        temp2(n,m) = -1  ! assign '-1' to the map


        ! check through the lower order processor ID
        ! since m is visited, starting from m-1
        do mm = m-1, 2, -1
          do nn = mm-1, 1, -1
            if (Pflag(mm) /= 0 .and. Pflag(nn) /= 0) then
              if (temp2(mm,nn) == 1) then
                Pflag(mm) = 0      ! if the proc has already been
                Pflag(nn) = 0      ! vivisted, assign '0'
                temp1( k,mm) = nn  ! for avaliable links
                temp1( k,nn) = mm  ! assign same task number
                temp2(mm,nn) = -1
                temp2(nn,mm) = -1
  
              end if
            end if
          end do
        end do
        
      end if
      
    end do
  end do

  Ntask = k

  ! write out the task for checking..
  !do i = 1, Ntask
  !  write(*,*) (temp1(i,j),j = NP, 1, -1)
  !end do

  allocate(task(Ntask,NP), stat=ier)
  if(ier /= 0) stop 'Allocation Error in "gen_lwork"'

  ! initialize task with '-1
  task = -1

  ! assign task from temp1 with correct dimensions
  task(:,:) = temp1(1:Ntask,:)

  !do i = 1, Ntask
  !  write(*,*) (task(i,j),j=1,NP)
  !end do

  ! loop through all processors to write the lwork files
  do p = 1, NP

    UMF(p) = 30 + p
    fname = trim('lwork'// trim(cname(p))//'.dat')
    open(UMF(p), file = fname, status = 'unknown')

    ! calculate the data length
    datalen = 1
    do i = 1, Ntask
      datalen = datalen + 4
      if (task(i,p) /= 0) then
        do j = 1, NOvNd(p,task(i,p))
          datalen = datalen + 2
        end do
      end if
    end do

    msgnum  = 100  ! message number
    msglen  = 1    ! message length
    write(UMF(p),210) Ntask, datalen
    do i = 1, Ntask
      RelPr = task(i,p)
      if (RelPr == 0) then
        write(UMF(p),220) msgnum
        write(UMF(p),220) 0
        write(UMF(p),220) -1
        write(UMF(p),220) 0
      else
        write(UMF(p),220) msgtag(i,p)
!        write(UMF(p),220) msgnum            ! message number
        write(UMF(p),220) CoMap(p,RelPr)    ! send or recv
        write(UMF(p),220) RelPr             ! processor ID
        write(UMF(p),220) NOvNd(p,RelPr)    ! number of msg's
        do j = 1, NOvNd(p,RelPr)
!          write(UMF(p),220) G2LNo(RelNd(p,RelPr,j),p)
          write(UMF(p),220) RelNd(p,RelPr,j)
          write(UMF(p),220) msglen
        end do
      end if
    end do
  end do
  
  210 format (2I8)
  220 format (I8)

  do i = 1, NP
    close(UMF(i))
  end do

  deallocate(task)

end subroutine gen_lwork
