c     This routine generates the new array of periodicities, new_IPER.
c       It is set up such that the master has a higher node number than
c       its slave, unless a node has no master, in which case we have
c       new_IPER(i) = i.     
c
c     new_IPER(i) = j indicates that j is the master of i.
c      
c       Feb 02, 2004      
c       J. Austin Cottrell
c       CAM Graduate Student
c       Institute for Computational Engineering Science
c       The University of Texas at Austin

      
      subroutine gen_new_IPER_3D(new_NNODZ, new_INN, new_U_KNOT,
     &     new_V_KNOT,new_W_KNOT, new_MCP,new_NCP,new_OCP,
     &     new_B_NET,new_IBC,new_IPER)

      use aAdjKeep

      include "common.h"



      
      integer new_NNODZ, new_INN(new_NNODZ,NSD), new_IPER(new_NNODZ),
     &     i, j, new_MCP, new_NCP, ri,rj,ci,cj,new_IBC(new_NNODZ,NSD),
     &     new_OCP,rk,ck
      real*8 tol, new_U_KNOT(new_MCP+P+1), new_V_KNOT(new_NCP+Q+1),
     &     rP(NSD), cP(NSD), new_B_NET(new_MCP,new_NCP,new_OCP,NSD+1),
     &     dist_inf,new_W_KNOT(new_OCP+R+1)



c     Initialize
      do i = 1,new_NNODZ
         new_IPER(i) = i
      enddo


c
c...  We have adopted the convention of, in all master/slave relation-
c       ships, calling the node with the higher number the master of
c       the node with the smaller number. Hence we loop backwards
c       from the higher numbered nodes downwards. Here we only check
c       to see that two nodes occupy the same point in physical space
c       and take that to be sufficient evidence of a master/slave pair.
c       This ensures that all master/slave pairs from the original mesh
c       are preserved, as well as any new pairs arising from refinement.
c     

c     A tolerance has been hardwired in. Adjust as needed.
      tol = 1d-8
      
c     Loop over "reference node"
c$$$      do i = new_NNODZ,1,-1
c$$$        
c$$$c...     Get NURBS coordinates for reference node
c$$$         ri = new_INN(i,1)
c$$$         rj = new_INN(i,2)
c$$$         rk = new_INN(i,3)
c$$$
c$$$c...     Get physical coordinates for reference node
c$$$         rP(1) = new_B_NET(ri,rj,rk,1)
c$$$         rP(2) = new_B_NET(ri,rj,rk,2)
c$$$         rP(3) = new_B_NET(ri,rj,rk,3)
c$$$
c$$$c...     Loop over "comparison node"
c$$$         do j = i-1,1,-1
c$$$
c$$$c...        Get NURBS coordinates for comparison node
c$$$            ci = new_INN(j,1)  
c$$$            cj = new_INN(j,2)
c$$$            ck = new_INN(j,3)
c$$$
c$$$c...        Get physical coordinates of comparison node
c$$$            cP(1) = new_B_NET(ci,cj,ck,1)
c$$$            cP(2) = new_B_NET(ci,cj,ck,2)
c$$$            cP(3) = new_B_NET(ci,cj,ck,3)
c$$$
c$$$
c$$$            dist_inf = abs(rP(1) - cP(1)) +
c$$$     &           abs(rP(2) - cP(2))+abs(rP(3) - cP(3))
c$$$
c$$$
c$$$c...     Check distance between points against the tolerance
c$$$            if (dist_inf.lt.tol) then
c$$$               new_IPER(j) = new_IPER(i)
c$$$               new_IBC(j,1) = 3 ! set indicator of slave to
c$$$               new_IBC(j,2) = 3 !  denote periodic BC
c$$$               new_IBC(j,3) = 3
c$$$            endif
c$$$
c$$$         enddo
c$$$      enddo


      return
      end
   
