
      subroutine output_3D_2(U_KNOT,V_KNOT,W_KNOT,B_NET,IPER,IBC,
     &     DIR_BC,IBC_FACE,LD_FACE,LD_EL,EY,NU)

c------------------------------------------------------------------------c
c                                                                        c
c        Subroutine to open file and read vertex, edge                   c
c        and element numbers. Initialze parameters of interest.          c
c                                                                        c
c        3D Version                                                      c
c                                                                        c
c                                                                        c
c        Nov 15, 2003                                                    c
c                                                                        c
c        J. Austin Cottrell                                              c
c        CES Graduate Student                                            c
c        Institute for Computational Engineering Science                 c
c        The University of Texas at Austin                               c
c                                                                        c
c------------------------------------------------------------------------c

  

      include "common.h"

      integer i, j, k,l, meshf
      real*8 U_KNOT(MCP+P+1),V_KNOT(NCP+Q+1),W_KNOT(OCP+R+1),
     &     B_NET(MCP,NCP,OCP,NSD+1),DIR_BC(NNODZ,NSD),
     &     LD_FACE(NFACE,NSD), LD_EL(NEL,NSD),EY(NEL),NU(NEL)
      integer IPER(NNODZ),IBC(NNODZ,NSD), IBC_FACE(NFACE,NSD)



      meshf = 12

c
c...  Read in preliminary information
c
      open (meshf, file='refined_mesh.dat', status='unknown')
      write (meshf,*)  NSD         ! number of spatial dimensions
      write (meshf,"(I5)", ADVANCE = "NO") P
      write (meshf,"(I5)", ADVANCE = "NO") Q
      write (meshf,"(I5)", ADVANCE = "YES") R ! degree of curves in u, v, and w direction
      write (meshf,"(I5)", ADVANCE = "NO") MCP
      write (meshf,"(I5)", ADVANCE = "NO") NCP
      write (meshf,"(I5)", ADVANCE = "YES") OCP ! number of control points in each direction



c
c...  Write knot vectors and control net
c
      do i = 1,MCP+P+1
         if (i < MCP+P+1) then
            write (meshf,"(F13.9)", ADVANCE = "NO") U_KNOT(i) ! write U_KNOT
         else
            write (meshf,"(F13.9)", ADVANCE = "YES") U_KNOT(i) 
         endif
      enddo

      do i = 1,NCP+Q+1
         if (i < NCP+Q+1) then
            write (meshf,"(F13.9)", ADVANCE = "NO") V_KNOT(i) ! write V_KNOT
         else
            write (meshf,"(F13.9)", ADVANCE = "YES") V_KNOT(i) 
         endif
      enddo

      do i = 1,OCP+R+1
         if (i < OCP+R+1) then
            write (meshf,"(F13.9)", ADVANCE = "NO") W_KNOT(i) ! write W_KNOT
         else
            write (meshf,"(F13.9)", ADVANCE = "YES") W_KNOT(i) 
         endif
      enddo

      do k = 1,(OCP)
         do j = 1,(NCP)
            do i = 1,(MCP)
               do l = 1,NSD+1
                  if (l < NSD+1) then
                     write (meshf,"(F16.9)", ADVANCE = "NO")
     &                    B_NET(i,j,k,l)
                  else
                     write (meshf,"(F16.9)", ADVANCE = "YES") 
     &                    B_NET(i,j,k,l)
                  endif
               enddo
            enddo
         enddo
      enddo

      !For Jessica's mesh
c      do k = 1,(OCP)
c         do j = 1,(NCP)
c            do i = 1,(MCP)
c               do l = 1,NSD+1
c                  if (l < NSD+1) then
c                     write (meshf,"(E20.12)", ADVANCE = "NO")
c     &                    B_NET(i,j,k,l)
c                  else
c                     write (meshf,"(E20.12)", ADVANCE = "YES") 
c     &                    B_NET(i,j,k,l)
c                  endif
c               enddo
c            enddo
c         enddo
c      enddo


      
c
c...  Write in Master/Slave relationships. These arise from periodic boundary
c       conditions, corners in the geometry, or any other case where two
c       control points must always remain equal.
c

      do i = 1, NNODZ
         write (meshf,*)  IPER(i) 
      enddo

c
c...  Write in Boundary/Face information
c
      write (meshf,"(I2)", ADVANCE = "NO") CLOSED_U_flag
      write (meshf,"(I2)", ADVANCE = "NO") CLOSED_V_flag
      write (meshf,"(I2)", ADVANCE = "YES") CLOSED_W_flag
                                ! flag determines if surface is closed
                                !  in some direction



c...  Write in boundary condition indicator at each node, in each direction
c     0 - nothing, 1 - Dir., 2 - Neu., 3 - Periodic
      do i = 1,NNODZ
         write (meshf,'(I2)', ADVANCE = "NO") IBC(i,1)
         write (meshf,'(I2)', ADVANCE = "NO") IBC(i,2)
         write (meshf,'(I2)', ADVANCE = "YES") IBC(i,3)
      enddo


c...  Write in Nodal displacements for inhomogeneous Dirichlet nodes.
      do i = 1, NNODZ
         write (meshf,"(F13.9)", ADVANCE = "NO") DIR_BC(i,1)
         write (meshf,"(F13.9)", ADVANCE = "NO") DIR_BC(i,2)
         write (meshf,"(F13.9)", ADVANCE = "YES") DIR_BC(i,3)
      enddo


c...  Write in boundary condition indicator on each face, in each direction
c     0 - nothing, 1 - constraint, 2 - load
      do i = 1,NFACE
         write (meshf,'(I2)', ADVANCE = "NO") IBC_FACE(i,1)
         write (meshf,'(I2)', ADVANCE = "NO") IBC_FACE(i,2)
         write (meshf,'(I2)', ADVANCE = "YES") IBC_FACE(i,3)
      enddo


c...  Write in actual load data on each face
c      This assumes (for the time being) that the load is contant
c      across each face. This will be modified in the future!
      do i = 1,NFACE
         write (meshf,"(F13.9)", ADVANCE = "NO") LD_FACE(i,1)
         write (meshf,"(F13.9)", ADVANCE = "NO") LD_FACE(i,2)
         write (meshf,"(F13.9)", ADVANCE = "YES") LD_FACE(i,3)
      enddo


     
c
c...  Write in body force loads on elements. This assumes constant
c       force across each element. We can modify this in the future.
      do i = 1,NEL
         write (meshf,"(F13.9)", ADVANCE = "NO") LD_EL(i,1)
         write (meshf,"(F13.9)", ADVANCE = "NO") LD_EL(i,2)
         write (meshf,"(F13.9)", ADVANCE = "YES") LD_EL(i,3)
      enddo

      
c
c...  Write in material data
c

c...  Young's Modulus in each element
      do i = 1,NEL
         write (meshf,*) EY(i)
      enddo

c... Poisson's ratio in each element
      do i = 1,NEL
         write (meshf,"(F13.9)") NU(i)
      enddo
         




      return
      end
