      subroutine RefineKnotVectSolid_3D(ml,pl,u_knotl,nl,ql,
     &     v_knotl,ol,rl,w_knotl,Pw,x_knot,rr1,rr2,rr3,dir,
     &     Qw,Ubar,Vbar,Wbar)


c
c purpose: This function inserts the knots in x_knot into either u_knot (if dir = 1),
c           v_knot (dir = 2), or w_knot (dir = 3) and refines the projective control 
c           points Pw such that neither the solid nor its parametrization changes.
c
c           NOTE: This could be cleverly rewritten so that, rather than call the surface
c                  refinement routine repeatedly, it would reuse certain coefficients
c                  that will be recalculated this way. If speed becomes and issue, I will
c                  rewrite this routine.
c
c
c January 31, 2004
c J. Austin Cottrell
c CES Graduate Student
c Texas Institute for Computational Engineering Science
c University of Texas at Austin



      include "common.h"

      
      integer ml,pl,nl,ql,ol,rl,rr,ucount,vcount,wcount,rr1,rr2,rr3,dir
      real*8 u_knotl(ml+pl+1),v_knotl(nl+ql+1),w_knotl(ol+rl+1), 
     &     Pw(ml,nl,ol,NSD+1), x_knot(max(rr1,rr2,rr3)),
     &     Ubar(ml+pl+1+rr1),Vbar(nl+ql+1+rr2), Wbar(ol+rl+1+rr3),
     &     Qw(ml+rr1,nl+rr2,ol+rr3,NSD+1)
      real*8, allocatable :: temp1(:,:,:),temp2(:,:,:)


c
c...  Refinement in u direction
c
      if (dir.eq.1) then
         
         rr = rr1
         
         allocate(temp1(ml,nl,NSD+1))
         allocate(temp2(ml+rr,nl,NSD+1))

         Qw = 0d+0
         temp1 = 0d+0
         temp2 = 0d+0

         do wcount = 1,ol

            do ucount = 1,ml
               do vcount = 1,nl
                  temp1(ucount,vcount,:) = Pw(ucount,vcount,wcount,:)
               enddo
            enddo


            call RefineKnotVectSurf(ml,pl,u_knotl,nl,ql,v_knotl,
     &           temp1,x_knot,rr,1,temp2,Ubar,Vbar)

            
            do ucount = 1,ml+rr
               do vcount = 1,nl
                  Qw(ucount,vcount,wcount,:) = temp2(ucount,vcount,:)
               enddo
            enddo

         enddo


c
c...  Refinement in v direction
c      

      elseif (dir.eq.2) then
         
         rr = rr2

         allocate(temp1(ml,nl,NSD+1))
         allocate(temp2(ml,nl+rr,NSD+1))

         Qw = 0d+0
         temp1 = 0d+0
         temp2 = 0d+0

         do wcount = 1,ol

            do ucount = 1,ml
               do vcount = 1,nl
                  temp1(ucount,vcount,:) = Pw(ucount,vcount,wcount,:)
               enddo
            enddo


            call RefineKnotVectSurf(ml,pl,u_knotl,nl,ql,v_knotl,
     &           temp1,x_knot,rr,0,temp2,Ubar,Vbar)

            
            do ucount = 1,ml
               do vcount = 1,nl+rr
                  Qw(ucount,vcount,wcount,:) = temp2(ucount,vcount,:)
               enddo
            enddo

         enddo



c
c...  Refinement in w direction
c      

      elseif (dir.eq.3) then
         
         rr = rr3

         allocate(temp1(ol,ml,NSD+1))
         allocate(temp2(ol+rr,ml,NSD+1))

         Qw = 0d+0
         temp1 = 0d+0
         temp2 = 0d+0

         do vcount = 1,nl

            do wcount = 1,ol
               do ucount = 1,ml
                  temp1(wcount,ucount,:) = Pw(ucount,vcount,wcount,:)
               enddo
            enddo


            call RefineKnotVectSurf(ol,rl,w_knotl,ml,pl,u_knotl,
     &           temp1,x_knot,rr,1,temp2,Wbar,Ubar)

            
            do wcount = 1,ol+rr
               do ucount = 1,ml
                  Qw(ucount,vcount,wcount,:) = temp2(wcount,ucount,:)
               enddo
            enddo

         enddo
      endif


      return
      end



