c     This routine generates face data through inheritance from the
c       parent faces. Geneology information is stored in FACE_CON.
c       See gen_FACE_CON_3D.f for details.     
c
c       Feb. 10, 2004
c       J. Austin Cottrell      
c       CAM Graduate Student
c       Institute for Computational Engineering Science
c       The University of Texas at Austin
      
      
      subroutine gen_FACE_DATA_3D(new_NNODZ,new_NFACE,FACE_CON,
     &     new_FACE_OR, 
     &     new_FACE_IEN, new_IPER,  new_IBC, new_IBC_FACE, new_LD_FACE,
     &     new_MCP, new_NCP, new_OCP, new_INN, new_B_NET,new_U_KNOT,
     &     new_V_KNOT,new_W_KNOT,new_DIR_BC)


      use aAdjKeep
      include "common.h"

      integer f, new_NFACE, FACE_CON(new_NFACE),new_NNODZ, nshb,
     &     new_FACE_OR(new_NFACE), i,j, new_FACE_IEN(new_NFACE,
     &     (P+1)*(Q+1)*(R+1)),new_INN(new_NNODZ,NSD),k,
     &     new_IPER(new_NNODZ), new_MCP, new_NCP,new_OCP
      real*8 new_LD_FACE(new_NFACE,NSD),
     &     new_B_NET(new_MCP,new_NCP,new_OCP,NSD+1),
     &     new_DIR_BC(new_NNODZ,NSD)
      integer  new_IBC(new_NNODZ,NSD), new_IBC_FACE(new_NFACE,NSD)
      real*8 new_U_KNOT(new_MCP+P+1), 
     &     new_V_KNOT(new_NCP+Q+1),new_W_KNOT(new_OCP+R+1)


      new_IBC = 0
      new_IBC_FACE = 0
      new_LD_FACE = 0d+0
      new_DIR_BC = 0d+0



      return
      end

      
      
      subroutine gen_FACE_DATA_3D_orig(new_NNODZ,new_NFACE,FACE_CON,
     &     new_FACE_OR, 
     &     new_FACE_IEN, new_IPER,  new_IBC, new_IBC_FACE, new_LD_FACE,
     &     new_MCP, new_NCP, new_OCP, new_INN, new_B_NET,new_U_KNOT,
     &     new_V_KNOT,new_W_KNOT,new_DIR_BC)

      use aAdjKeep
      include "common.h"

      integer f, new_NFACE, FACE_CON(new_NFACE),new_NNODZ, nshb,
     &     new_FACE_OR(new_NFACE), i,j, new_FACE_IEN(new_NFACE,
     &     (P+1)*(Q+1)*(R+1)),new_INN(new_NNODZ,NSD),k,
     &     new_IPER(new_NNODZ), new_MCP, new_NCP,new_OCP
      real*8 new_LD_FACE(new_NFACE,NSD),
     &     new_B_NET(new_MCP,new_NCP,new_OCP,NSD+1),
     &     new_DIR_BC(new_NNODZ,NSD)
      integer  new_IBC(new_NNODZ,NSD), new_IBC_FACE(new_NFACE,NSD)
      real*8 new_U_KNOT(new_MCP+P+1), 
     &     new_V_KNOT(new_NCP+Q+1),new_W_KNOT(new_OCP+R+1)




c...  Allocations and initialization
      new_IBC = 0
      new_IBC_FACE = 0
      new_LD_FACE = 0d+0
      new_DIR_BC = 0d+0



c
c...  First set periodicities. These may arise from a closed net
c       or from corners and sharp edges in physical space. As this
c       process is a bit involved, it has been placed in a separate
c       subroutine
      call gen_new_IPER_3D(new_NNODZ, new_INN, new_U_KNOT, new_V_KNOT,
     &     new_W_KNOT,new_MCP, new_NCP, new_OCP,new_B_NET, new_IBC,
     &     new_IPER)



      
c
c...  Now loop over faces and check boundary information
c

   
      
c...  loop over edges
      do f = 1, new_NFACE

        

c...  Check for Neumann conditions. This is purely a face phenomenon,
c      so we do not need modify new_IBC, only new_IBC_FACE.
         if (IBC_FACE(FACE_CON(f),1).eq.2) then
            new_IBC_FACE(f,1) = 2
         endif
         if (IBC_FACE(FACE_CON(f),2).eq.2) then
            new_IBC_FACE(f,2) = 2
         endif
         if (IBC_FACE(FACE_CON(f),3).eq.2) then
            new_IBC_FACE(f,3) = 2
         endif



c...  Now check for Dirichlet conditions. Due to the fact that
c      basis functions are not completely localized, this may
c      declare some nodes constrained which were formerly set
c      to be periodic. This is okay as the condition of being 
c      constrained takes precedence over any other conditions.

c...  Location of nodes to be constrained depends on orientation
c        though the basic behavior of each loop is the same.
         
c        ORIENTATION 1
         if (new_FACE_OR(f).eq.1) then
            if (IBC_FACE(FACE_CON(f),1).eq.1) then
               new_IBC_FACE(f,1) = 1
               do i = (P+1)*(Q+1)*R+1,(P+1)*(Q+1)*(R+1)
                  new_IBC(new_FACE_IEN(f,i),1) = 1
                  new_IBC(new_IPER(new_FACE_IEN(f,i)),1) = 1
                                ! if a slave is constrained, the master
                                !    must be as well.
                  new_DIR_BC(new_FACE_IEN(f,i),1) =
     &                DIR_BC(FACE_IEN(FACE_CON(f),(P+1)*(Q+1)*(R+1)),1)
               enddo
            endif
            if (IBC_FACE(FACE_CON(f),2).eq.1) then
               new_IBC_FACE(f,2) = 1
               do i = (P+1)*(Q+1)*R+1,(P+1)*(Q+1)*(R+1)
                  new_IBC(new_FACE_IEN(f,i),2) = 1
                  new_IBC(new_IPER(new_FACE_IEN(f,i)),2) = 1
                                ! if a slave is constrained, the master
                                !    must be as well.
                  new_DIR_BC(new_FACE_IEN(f,i),2) =
     &                DIR_BC(FACE_IEN(FACE_CON(f),(P+1)*(Q+1)*(R+1)),2)
               enddo
            endif
            if (IBC_FACE(FACE_CON(f),3).eq.1) then
               new_IBC_FACE(f,3) = 1
               do i = (P+1)*(Q+1)*R+1,(P+1)*(Q+1)*(R+1)
                  new_IBC(new_FACE_IEN(f,i),3) = 1
                  new_IBC(new_IPER(new_FACE_IEN(f,i)),3) = 1
                                ! if a slave is constrained, the master
                                !    must be as well.
                  new_DIR_BC(new_FACE_IEN(f,i),3) =
     &                DIR_BC(FACE_IEN(FACE_CON(f),(P+1)*(Q+1)*(R+1)),3)
               enddo
            endif
         endif


c...  ORIENTATION 2
         if (new_FACE_OR(f).eq.2) then
            if (IBC_FACE(FACE_CON(f),1).eq.1) then
               new_IBC_FACE(f,1) = 1
               do j = (P+1)*(Q+1),(P+1)*(Q+1)*(R+1),(P+1)*(Q+1)
                  do i = 0,P
                     new_IBC(new_FACE_IEN(f,j-i),1) = 1
                     new_IBC(new_IPER(new_FACE_IEN(f,j-i)),1) = 1
                                ! if a slave is constrained, the master
                                !    must be as well.
                     new_DIR_BC(new_FACE_IEN(f,j-i),1) =
     &                    DIR_BC(FACE_IEN(FACE_CON(f),(P+1)*(Q+1)),1)
                  enddo
               enddo
            endif
            if (IBC_FACE(FACE_CON(f),2).eq.1) then
               new_IBC_FACE(f,2) = 1
               do j = (P+1)*(Q+1),(P+1)*(Q+1)*(R+1),(P+1)*(Q+1)
                  do i = 0,P
                     new_IBC(new_FACE_IEN(f,j-i),2) = 1
                     new_IBC(new_IPER(new_FACE_IEN(f,j-i)),2) = 1
                                ! if a slave is constrained, the master
                                !    must be as well.
                     new_DIR_BC(new_FACE_IEN(f,j-i),2) =
     &                    DIR_BC(FACE_IEN(FACE_CON(f),(P+1)*(Q+1)),2)
                  enddo
               enddo
            endif
            if (IBC_FACE(FACE_CON(f),3).eq.1) then
               new_IBC_FACE(f,3) = 1
               do j = (P+1)*(Q+1),(P+1)*(Q+1)*(R+1),(P+1)*(Q+1)
                  do i = 0,P
                     new_IBC(new_FACE_IEN(f,j-i),3) = 1
                     new_IBC(new_IPER(new_FACE_IEN(f,j-i)),3) = 1
                                ! if a slave is constrained, the master
                                !    must be as well.
                     new_DIR_BC(new_FACE_IEN(f,j-i),3) =
     &                    DIR_BC(FACE_IEN(FACE_CON(f),(P+1)*(Q+1)),3)
                  enddo
               enddo
            endif
         endif



c...  ORIENTATION 3
         if (new_FACE_OR(f).eq.3) then
            if (IBC_FACE(FACE_CON(f),1).eq.1) then
               new_IBC_FACE(f,1) = 1
               do j = 0,(P+1)*(Q+1)*(R),(P+1)*(Q+1)
                  do i = 1,(P+1)*Q+1,(P+1)
                     new_IBC(new_FACE_IEN(f,j+i),1) = 1
                     new_IBC(new_IPER(new_FACE_IEN(f,j+i)),1) = 1
                                ! if a slave is constrained, the master
                                !    must be as well.
                     new_DIR_BC(new_FACE_IEN(f,j+i),1) =
     &                    DIR_BC(FACE_IEN(FACE_CON(f),1),1)
                  enddo
               enddo
            endif
            if (IBC_FACE(FACE_CON(f),2).eq.1) then
               new_IBC_FACE(f,2) = 1
               do j = 0,(P+1)*(Q+1)*(R),(P+1)*(Q+1)
                  do i = 1,(P+1)*Q+1,(P+1)
                     new_IBC(new_FACE_IEN(f,j+i),2) = 1
                     new_IBC(new_IPER(new_FACE_IEN(f,j+i)),2) = 1
                                ! if a slave is constrained, the master
                                !    must be as well.
                     new_DIR_BC(new_FACE_IEN(f,j+i),2) =
     &                    DIR_BC(FACE_IEN(FACE_CON(f),1),2)
                  enddo
               enddo
            endif
            if (IBC_FACE(FACE_CON(f),3).eq.1) then
               new_IBC_FACE(f,3) = 1
               do j = 0,(P+1)*(Q+1)*(R),(P+1)*(Q+1)
                  do i = 1,(P+1)*Q+1,(P+1)
                     new_IBC(new_FACE_IEN(f,j+i),3) = 1
                     new_IBC(new_IPER(new_FACE_IEN(f,j+i)),3) = 1
                                ! if a slave is constrained, the master
                                !    must be as well.
                     new_DIR_BC(new_FACE_IEN(f,j+i),3) =
     &                    DIR_BC(FACE_IEN(FACE_CON(f),1),3)
                  enddo
               enddo
            endif
         endif




c...  ORIENTATION 4
         if (new_FACE_OR(f).eq.4) then
            if (IBC_FACE(FACE_CON(f),1).eq.1) then
               new_IBC_FACE(f,1) = 1
               do j = 0,(P+1)*(Q+1)*R,(P+1)*(Q+1)
                  do i = 1,P+1
                     new_IBC(new_FACE_IEN(f,j+i),1) = 1
                     new_IBC(new_IPER(new_FACE_IEN(f,j+i)),1) = 1
                                ! if a slave is constrained, the master
                                !    must be as well.
                     new_DIR_BC(new_FACE_IEN(f,j+i),1) =
     &                    DIR_BC(FACE_IEN(FACE_CON(f),1),1)
                  enddo
               enddo
            endif
            if (IBC_FACE(FACE_CON(f),2).eq.1) then
               new_IBC_FACE(f,2) = 1
               do j = 0,(P+1)*(Q+1)*(R),(P+1)*(Q+1)
                  do i = 1,P+1
                     new_IBC(new_FACE_IEN(f,j+i),2) = 1
                     new_IBC(new_IPER(new_FACE_IEN(f,j+i)),2) = 1
                                ! if a slave is constrained, the master
                                !    must be as well.
                     new_DIR_BC(new_FACE_IEN(f,j+i),2) =
     &                    DIR_BC(FACE_IEN(FACE_CON(f),1),2)
                  enddo
               enddo
            endif
            if (IBC_FACE(FACE_CON(f),3).eq.1) then
               new_IBC_FACE(f,3) = 1
               do j = 0,(P+1)*(Q+1)*R,(P+1)*(Q+1)
                  do i = 1,P+1
                     new_IBC(new_FACE_IEN(f,j+i),3) = 1
                     new_IBC(new_IPER(new_FACE_IEN(f,j+i)),3) = 1
                                ! if a slave is constrained, the master
                                !    must be as well.
                     new_DIR_BC(new_FACE_IEN(f,j+i),3) =
     &                    DIR_BC(FACE_IEN(FACE_CON(f),1),3)
                  enddo
               enddo
            endif
         endif



c...  ORIENTATION 5
         if (new_FACE_OR(f).eq.5) then
            if (IBC_FACE(FACE_CON(f),1).eq.1) then
               new_IBC_FACE(f,1) = 1
               do j = 0,(P+1)*(Q+1)*(R),(P+1)*(Q+1)
                  do i = P+1,(P+1)*(Q+1),(P+1)
                     new_IBC(new_FACE_IEN(f,j+i),1) = 1
                     new_IBC(new_IPER(new_FACE_IEN(f,j+i)),1) = 1
                                ! if a slave is constrained, the master
                                !    must be as well.
                     new_DIR_BC(new_FACE_IEN(f,j+i),1) =
     &                    DIR_BC(FACE_IEN(FACE_CON(f),P+1),1)
                  enddo
               enddo
            endif
            if (IBC_FACE(FACE_CON(f),2).eq.1) then
               new_IBC_FACE(f,2) = 1
               do j = 0,(P+1)*(Q+1)*(R),(P+1)*(Q+1)
                  do i = P+1,(P+1)*(Q+1),(P+1)
                     new_IBC(new_FACE_IEN(f,j+i),2) = 1
                     new_IBC(new_IPER(new_FACE_IEN(f,j+i)),2) = 1
                                ! if a slave is constrained, the master
                                !    must be as well.
                     new_DIR_BC(new_FACE_IEN(f,j+i),2) =
     &                    DIR_BC(FACE_IEN(FACE_CON(f),P+1),2)
                  enddo
               enddo
            endif
            if (IBC_FACE(FACE_CON(f),3).eq.1) then
               new_IBC_FACE(f,3) = 1
               do j = 0,(P+1)*(Q+1)*(R),(P+1)*(Q+1)
                  do i = P+1,(P+1)*(Q+1),(P+1)
                     new_IBC(new_FACE_IEN(f,j+i),3) = 1
                     new_IBC(new_IPER(new_FACE_IEN(f,j+i)),3) = 1
                                ! if a slave is constrained, the master
                                !    must be as well.
                     new_DIR_BC(new_FACE_IEN(f,j+i),3) =
     &                    DIR_BC(FACE_IEN(FACE_CON(f),P+1),3)
                  enddo
               enddo
            endif
         endif



c...  ORIENTATION 6
         if (new_FACE_OR(f).eq.6) then
            if (IBC_FACE(FACE_CON(f),1).eq.1) then
               new_IBC_FACE(f,1) = 1
               do i = 1,(P+1)*(Q+1)
                  new_IBC(new_FACE_IEN(f,i),1) = 1
                  new_IBC(new_IPER(new_FACE_IEN(f,i)),1) = 1
                                ! if a slave is constrained, the master
                                !    must be as well.
                  new_DIR_BC(new_FACE_IEN(f,i),1) =
     &                DIR_BC(FACE_IEN(FACE_CON(f),1),1)
               enddo
            endif
            if (IBC_FACE(FACE_CON(f),2).eq.1) then
               new_IBC_FACE(f,2) = 1
               do i = 1,(P+1)*(Q+1)
                  new_IBC(new_FACE_IEN(f,i),2) = 1
                  new_IBC(new_IPER(new_FACE_IEN(f,i)),2) = 1
                                ! if a slave is constrained, the master
                                !    must be as well.
                  new_DIR_BC(new_FACE_IEN(f,i),2) =
     &                DIR_BC(FACE_IEN(FACE_CON(f),1),2)
               enddo
            endif
            if (IBC_FACE(FACE_CON(f),3).eq.1) then
               new_IBC_FACE(f,3) = 1
               do i = 1,(P+1)*(Q+1)
                  new_IBC(new_FACE_IEN(f,i),3) = 1
                  new_IBC(new_IPER(new_FACE_IEN(f,i)),3) = 1
                                ! if a slave is constrained, the master
                                !    must be as well.
                  new_DIR_BC(new_FACE_IEN(f,i),3) =
     &                DIR_BC(FACE_IEN(FACE_CON(f),1),3)
               enddo
            endif
         endif




c
c...  While looping over faces, assign load data. This assumes that the
c      load is constant. Nonconstant loads are dealt with in the analysis
c      code by modifications in IntElmAss_3D.f. If a nonconstant load is
c      present, LD_FACE should equal zero for that face.         
c
         new_LD_FACE(f,1) = LD_FACE(FACE_CON(f),1)
         new_LD_FACE(f,2) = LD_FACE(FACE_CON(f),2)
         new_LD_FACE(f,3) = LD_FACE(FACE_CON(f),3)



      enddo




c
c...  Lastly, we must loop through the nodes a final time to finish
c       our new_IBC array. At this stage it is possible that some
c       "master" nodes are constrained, while their "slaves" are only
c       labeled as periodic. We must identify these slaves as constrained
c       if we are to generate the appropriate stiffness matrices during 
c       analysis. 
c     
      do i = 1, new_NNODZ
         if (new_IBC(new_IPER(i),1).eq.1) then
            new_IBC(i,1) = 1
         endif
         if (new_IBC(new_IPER(i),2).eq.1) then
            new_IBC(i,2) = 1
         endif
         if (new_IBC(new_IPER(i),3).eq.1) then
            new_IBC(i,3) = 1
         endif
      enddo




      return
      end





