

      use aAdjKeep
      include "common.h"


!
!
!     h-refinement for the 3D code.
!
!     January 30, 2004
!     modified:  March 16, 2004 -  anisotropic refinement
!      
!       J. Austin Cottrell
!       CAM Graduate Student
!       Institute for Computational Engineering Science
!       The University of Texas at Austin  


c     -------------------------------------------------------
      integer i, icount, new_MCP, new_NCP, new_OCP, new_NNODZ, new_NEL,
     &     new_NFACE, jcount, j, kcount,l,k,uflag,vflag,wflag
      
      integer, allocatable :: new_IEN(:,:), new_INN(:,:), new_IBC(:,:),
     &     new_FACE_IEN(:,:), new_FACE_OR(:), new_IBC_FACE(:,:),
     &     FACE_CON(:), EL_CON(:), new_IPER(:)
      real*8, allocatable :: x_knot_u(:), x_knot_v(:),x_knot_w(:),
     &     new_U_KNOT(:),new_V_KNOT(:),new_W_KNOT(:),
     &     new_B_NET(:,:,:,:),
     &     new_BW(:,:,:,:), Pw_1(:,:), Pw_2(:,:),Pw_3(:,:), Pw_4(:,:),
     &     temp(:), temp2(:,:,:,:),new_LD_FACE(:,:), new_LD_EL(:,:),
     &     new_EY(:), new_NU(:),new_DIR_BC(:,:), temp3(:,:,:,:)
    
      real*8 u1,u2,u3,u4,v1,v2,v3,v4,w1,w2,w3,w4,
     &     c1_norm, c2_norm, c3_norm, c4_norm,a1(3), b1(3), a2(3), 
     &     b2(3), c1(2,3), c2(2,3),
     &     a3(3), b3(3), a4(3), b4(3), c3(2,3), c4(2,3)
     
      character*10 inp     
c     -------------------------------------------------------

c...  Get initial mesh data
      call input_3D
      
      allocate(INN(NNODZ,3))
      NSHL = (P+1)*(Q+1)*(R+1)
      allocate(IEN(NEL,NSHL))
c...  moved from genIEN_INN_3D for to fix bugs
      
c...  Form projective control net
      BW = B_NET
      do i = 1,NSD
         BW(:,:,:,i) = BW(:,:,:,i)*B_NET(:,:,:,NSD+1)
      enddo
      
      
c...  Figure out number of knots to be inserted in each direction
c     and new knot values
      
      i = iargc()
      if (i.eq.3) then  
        call getarg(1,inp); read(inp,*) uflag
        call getarg(2,inp); read(inp,*) vflag 
        call getarg(3,inp); read(inp,*) wflag 
      else
        write(*,*)"Refine in u direction? 1-yes, 0-no: "
        read(*,*) uflag
        write(*,*)"Refine in v direction? 1-yes, 0-no: "
        read(*,*) vflag      
        write(*,*)"Refine in w direction? 1-yes, 0-no: "
        read(*,*) wflag    
      endif
c     
c...  Begin by finding knots to be inserted into  U_KNOT 
c     
      allocate(temp(max(MCP-P,NCP-Q,OCP-R)))

      if (uflag.eq.1) then 
        write(*,*) "Resizing u-dir..."
      
        allocate(Pw_1(MCP,NSD+1))
        allocate(Pw_2(MCP,NSD+1))
        allocate(Pw_3(MCP,NSD+1))
        allocate(Pw_4(MCP,NSD+1))
        
        icount = 0
        temp = 0d+0
        do i = 1,MCP
          Pw_1(i,:) = BW(i,1,1,:)
          Pw_2(i,:) = BW(i,NCP,1,:)
          Pw_3(i,:) = BW(i,1,OCP,:)
          Pw_4(i,:) = BW(i,NCP,OCP,:)
        enddo
        
      

      
      
c...  Loop through elements in u direction
      
        do i = 1,MCP-P
          if (U_KNOT(P+i).ne.U_KNOT(P+i+1)) then
            icount = icount + 1
c           number of new knots in u is equal to the number of
c              nonzero knot spans.
                          
            
            
c...        Get endpoints of the 4 relevant chords
c             for this particular knot span            
            call rat_curve_point(i+P,P,MCP,
     &        U_KNOT(P+i),U_KNOT, Pw_1,a1)
            call rat_curve_point(i+P,P,MCP,
     &        U_KNOT(P+i+1),U_KNOT, Pw_1,b1)
            call rat_curve_point(i+P,P,MCP,
     &        U_KNOT(P+i),U_KNOT, Pw_2,a2)
            call rat_curve_point(i+P,P,MCP,
     &        U_KNOT(P+i+1),U_KNOT, Pw_2,b2)
            call rat_curve_point(i+P,P,MCP,
     &        U_KNOT(P+i),U_KNOT, Pw_3,a3)
            call rat_curve_point(i+P,P,MCP,
     &        U_KNOT(P+i+1),U_KNOT, Pw_3,b3)
            call rat_curve_point(i+P,P,MCP,
     &        U_KNOT(P+i),U_KNOT, Pw_4,a4)
            call rat_curve_point(i+P,P,MCP,
     &        U_KNOT(P+i+1),U_KNOT, Pw_4,b4)
            
            
            
c...        approximate parameter of midpoint of chords
c              this is "point inversion"
            call projection_curve((a1(:)+b1(:))/2.0,
     &        i+P,P,MCP,U_KNOT,Pw_1,u1)
            call projection_curve((a2(:)+b2(:))/2.0,
     &        i+P,P,MCP,U_KNOT,Pw_2,u2)
            call projection_curve((a3(:)+b3(:))/2.0,
     &        i+P,P,MCP,U_KNOT,Pw_3,u3)
            call projection_curve((a4(:)+b4(:))/2.0,
     &        i+P,P,MCP,U_KNOT,Pw_4,u4)            

            
c...        Take weighted average of the four parameter values. Base weight
c           on ratio of parameter derivatives.
            call rat_curvedervs(i+P,P,MCP,u1,U_KNOT, Pw_1,1,c1)
            call rat_curvedervs(i+P,P,MCP,u2,U_KNOT, Pw_2,1,c2)
            call rat_curvedervs(i+P,P,MCP,u3,U_KNOT, Pw_3,1,c3)
            call rat_curvedervs(i+P,P,MCP,u4,U_KNOT, Pw_4,1,c4)
            
            c1_norm = 0d+0
            c2_norm = 0d+0
            c3_norm = 0d+0
            c4_norm = 0d+0
            
            do j = 1,NSD
              c1_norm = c1_norm + c1(2,j)**2
              c2_norm = c2_norm + c2(2,j)**2
              c3_norm = c3_norm + c3(2,j)**2
              c4_norm = c4_norm + c4(2,j)**2
            enddo
            
            c1_norm = sqrt(c1_norm)
            c2_norm = sqrt(c2_norm)
            c3_norm = sqrt(c3_norm)
            c4_norm = sqrt(c4_norm)
            
            
            temp(icount) = ((c1_norm*u1 + c2_norm*u2 +
     &        c3_norm*u3 + c4_norm*u4 )/
     &        (c1_norm + c2_norm + c3_norm + c4_norm))
            
            
          endif
        enddo
        
        
        
        
        allocate(x_knot_u(icount))
        new_MCP = MCP+icount    ! new number of control points in u
        allocate(new_U_KNOT(new_MCP+P+1))
        x_knot_u(:) = temp(1:icount) ! new knots to be inserted in u
        
        
        deallocate(Pw_1)
        deallocate(Pw_2)
        deallocate(Pw_3)
        deallocate(Pw_4)
      else
        new_MCP = MCP
        allocate(new_U_KNOT(new_MCP+P+1))
      endif
      
c     ---------------------------------------------------
      
c     
c...  Now find new knots for V_KNOT
c     

      if (vflag.eq.1) then
        write(*,*) "Resizing v-dir..."
        allocate(Pw_1(NCP,NSD+1))
        allocate(Pw_2(NCP,NSD+1))
        allocate(Pw_3(NCP,NSD+1))
        allocate(Pw_4(NCP,NSD+1))
        
        jcount = 0
        temp = 0d+0
        do i = 1,NCP
          Pw_1(i,:) = BW(1,i,1,:)
          Pw_2(i,:) = BW(MCP,i,1,:)
          Pw_3(i,:) = BW(1,i,OCP,:)
          Pw_4(i,:) = BW(MCP,i,OCP,:)
        enddo
        
        
        
        
c...    Loop through elements in u direction
        
        do i = 1,NCP-Q
          if (V_KNOT(Q+i).ne.V_KNOT(Q+i+1)) then
            jcount = jcount + 1 ! number of new knots in v
            
            
            
c...        Get endpoints of chords
            call rat_curve_point(i+Q,Q,NCP,
     &        V_KNOT(Q+i),V_KNOT, Pw_1,a1)
            call rat_curve_point(i+Q,Q,NCP,
     &        V_KNOT(Q+i+1),V_KNOT, Pw_1,b1)
            call rat_curve_point(i+Q,Q,NCP,
     &        V_KNOT(Q+i),V_KNOT, Pw_2,a2)
            call rat_curve_point(i+Q,Q,NCP,
     &        V_KNOT(Q+i+1),V_KNOT, Pw_2,b2)
            call rat_curve_point(i+Q,Q,NCP,
     &        V_KNOT(Q+i),V_KNOT, Pw_3,a3)
            call rat_curve_point(i+Q,Q,NCP,
     &        V_KNOT(Q+i+1),V_KNOT, Pw_3,b3)
            call rat_curve_point(i+Q,Q,NCP,
     &        V_KNOT(Q+i),V_KNOT, Pw_4,a4)
            call rat_curve_point(i+Q,Q,NCP,
     &        V_KNOT(Q+i+1),V_KNOT, Pw_4,b4)
            
            
            
c...        find parameter of midpoint of chords
            call projection_curve((a1(:)+b1(:))/2.0,
     &        i+Q,Q,NCP,V_KNOT,Pw_1,v1)
            call projection_curve((a2(:)+b2(:))/2.0,
     &        i+Q,Q,NCP,V_KNOT,Pw_2,v2)
            call projection_curve((a3(:)+b3(:))/2.0,
     &        i+Q,Q,NCP,V_KNOT,Pw_3,v3)
            call projection_curve((a4(:)+b4(:))/2.0,
     &        i+Q,Q,NCP,V_KNOT,Pw_4,v4)            
            
c...        Take weighted average of the four parameter values. Base weight
c           on ratio of parameter derivatives.
            call rat_curvedervs(i+Q,Q,NCP,v1,V_KNOT, Pw_1,1,c1)
            call rat_curvedervs(i+Q,Q,NCP,v2,V_KNOT, Pw_2,1,c2)
            call rat_curvedervs(i+Q,Q,NCP,v3,V_KNOT, Pw_3,1,c3)
            call rat_curvedervs(i+Q,Q,NCP,v4,V_KNOT, Pw_4,1,c4)
            
            c1_norm = 0d+0
            c2_norm = 0d+0
            c3_norm = 0d+0
            c4_norm = 0d+0
            
            do j = 1,NSD
              c1_norm = c1_norm + c1(2,j)**2
              c2_norm = c2_norm + c2(2,j)**2
              c3_norm = c3_norm + c3(2,j)**2
              c4_norm = c4_norm + c4(2,j)**2
            enddo
            
            c1_norm = sqrt(c1_norm)
            c2_norm = sqrt(c2_norm)
            c3_norm = sqrt(c3_norm)
            c4_norm = sqrt(c4_norm)
            
            
            temp(jcount) = ((c1_norm*v1 + c2_norm*v2 +
     &        c3_norm*v3 + c4_norm*v4 )/
     &        (c1_norm + c2_norm + c3_norm + c4_norm))
            
            
          endif
        enddo
      
      
      
      
      
        allocate(x_knot_v(jcount))
        new_NCP = NCP+jcount    ! new number of control points in v
        allocate(new_V_KNOT(new_NCP+Q+1))
        x_knot_v(:) = temp(1:jcount) ! new knots to be inserted in v
      
      
      
        deallocate(Pw_1)
        deallocate(Pw_2)
        deallocate(Pw_3)
        deallocate(Pw_4)
      else
        new_NCP = NCP
        allocate(new_V_KNOT(new_NCP+Q+1))
      endif
      
c     ---------------------------------------------------------------
      
c     
c...  Now find new knots for W_KNOT
c     

      if (wflag.eq.1) then
        write(*,*) "Resizing w-dir..."
        allocate(Pw_1(OCP,NSD+1))
        allocate(Pw_2(OCP,NSD+1))
        allocate(Pw_3(OCP,NSD+1))
        allocate(Pw_4(OCP,NSD+1))
        
        kcount = 0
        temp = 0d+0
        do i = 1,OCP
          Pw_1(i,:) = BW(1,1,i,:)
          Pw_2(i,:) = BW(MCP,1,i,:)
          Pw_3(i,:) = BW(1,NCP,i,:)
          Pw_4(i,:) = BW(MCP,NCP,i,:)
        enddo
        
        
        
        
c...    Loop through elements in w direction
      
        do i = 1,OCP-R
          if (W_KNOT(R+i).ne.W_KNOT(R+i+1)) then
            kcount = kcount + 1 ! number of new knots in w
            
            
            
c...        Get endpoints of chords
            call rat_curve_point(i+R,R,OCP,
     &        W_KNOT(R+i),W_KNOT, Pw_1,a1)
            call rat_curve_point(i+R,R,OCP,
     &        W_KNOT(R+i+1),W_KNOT, Pw_1,b1)
            call rat_curve_point(i+R,R,OCP,
     &        W_KNOT(R+i),W_KNOT, Pw_2,a2)
            call rat_curve_point(i+R,R,OCP,
     &        W_KNOT(R+i+1),W_KNOT, Pw_2,b2)
            call rat_curve_point(i+R,R,OCP,
     &        W_KNOT(R+i),W_KNOT, Pw_3,a3)
            call rat_curve_point(i+R,R,OCP,
     &        W_KNOT(R+i+1),W_KNOT, Pw_3,b3)
            call rat_curve_point(i+R,R,OCP,
     &        W_KNOT(R+i),W_KNOT, Pw_4,a4)
            call rat_curve_point(i+R,R,OCP,
     &        W_KNOT(R+i+1),W_KNOT, Pw_4,b4)
            
            
            
c...        find parameter of midpoint of chords
            call projection_curve((a1(:)+b1(:))/2.0,
     &        i+R,R,OCP,W_KNOT,Pw_1,w1)
            call projection_curve((a2(:)+b2(:))/2.0,
     &        i+R,R,OCP,W_KNOT,Pw_2,w2)
            call projection_curve((a3(:)+b3(:))/2.0,
     &        i+R,R,OCP,W_KNOT,Pw_3,w3)
            call projection_curve((a4(:)+b4(:))/2.0,
     &        i+R,R,OCP,W_KNOT,Pw_4,w4)            
            
c...        Take weighted average of the four parameter values. Base weight
c           on ratio of parameter derivatives.
            call rat_curvedervs(i+R,R,OCP,w1,W_KNOT, Pw_1,1,c1)
            call rat_curvedervs(i+R,R,OCP,w2,W_KNOT, Pw_2,1,c2)
            call rat_curvedervs(i+R,R,OCP,w3,W_KNOT, Pw_3,1,c3)
            call rat_curvedervs(i+R,R,OCP,w4,W_KNOT, Pw_4,1,c4)
            
            c1_norm = 0d+0
            c2_norm = 0d+0
            c3_norm = 0d+0
            c4_norm = 0d+0
            
            do j = 1,NSD
              c1_norm = c1_norm + c1(2,j)**2
              c2_norm = c2_norm + c2(2,j)**2
              c3_norm = c3_norm + c3(2,j)**2
              c4_norm = c4_norm + c4(2,j)**2
            enddo
            
            c1_norm = sqrt(c1_norm)
            c2_norm = sqrt(c2_norm)
            c3_norm = sqrt(c3_norm)
            c4_norm = sqrt(c4_norm)
            
            
            temp(kcount) = ((c1_norm*w1 + c2_norm*w2 +
     &        c3_norm*w3 + c4_norm*w4 )/
     &        (c1_norm + c2_norm + c3_norm + c4_norm))
            
            
          endif
        enddo
        
      
      
      
        allocate(x_knot_w(kcount))
        new_OCP = OCP+kcount    ! new number of control points in w
        allocate(new_W_KNOT(new_OCP+R+1))
        x_knot_w(:) = temp(1:kcount) ! new knots to be inserted in w



        deallocate(Pw_1)
        deallocate(Pw_2)
        deallocate(Pw_3)
        deallocate(Pw_4)
        
      else
        new_OCP = OCP
        allocate(new_W_KNOT(new_OCP+R+1))
      endif
      deallocate(temp)
c     ------------------------------------------------------------
      
 
      allocate(new_B_NET(new_MCP,new_NCP,new_OCP,NSD+1))


      
      
c...  Set number of global nodes
      new_NNODZ = new_MCP*new_NCP*new_OCP
c...  Set number of elements
      new_NEL = (new_MCP-P)*(new_NCP-Q)*(new_OCP-R)
      new_NFACE = 2*(new_MCP-P)*(new_NCP-Q)*(1-CLOSED_W_flag) +
     &     2*(new_NCP-Q)*(new_OCP-R)*(1-CLOSED_U_flag) +
     &     2*(new_MCP-P)*(new_OCP-R)*(1-CLOSED_V_flag)
      allocate(new_FACE_IEN(new_NFACE,(P+1)*(Q+1)*(R+1)))
      allocate(new_FACE_OR(new_NFACE))
      allocate(FACE_CON(new_NFACE))
      allocate(EL_CON(new_NEL))
      allocate(new_LD_EL(new_NEL,NSD))
      allocate(new_EY(new_NEL))
      allocate(new_NU(new_NEL))
      allocate(new_INN(new_NNODZ,NSD))
      allocate(new_IEN(new_NEL,(P+1)*(Q+1)*(R+1)))
      allocate(new_IBC(new_NNODZ,NSD))
      allocate(new_IBC_FACE(new_NFACE,NSD))
      allocate(new_IPER(new_NNODZ))
      allocate(new_LD_FACE(new_NFACE,NSD))
      allocate(new_DIR_BC(new_NNODZ,NSD))
      
c     
c...  Form new Control Net in Projective coordinates
c     
      allocate(temp2(new_MCP,NCP,OCP,NSD+1))
      allocate(temp3(new_MCP,new_NCP,OCP,NSD+1))
      
      allocate(new_BW(new_MCP,new_NCP,new_OCP,NSD+1))
      temp2 = 0d+0
      new_BW = 0d+0
      
c...  refine first in u
      if (uflag.eq.1) then
        write(*,*) "Refining u-dir..."
        call RefineKnotVectSolid_3D(MCP,P,U_KNOT,NCP,Q,V_KNOT,OCP,R,
     &    W_KNOT,BW,x_knot_u,icount,0,0,1,temp2,new_U_KNOT,
     &    V_KNOT,W_KNOT)
      else
        temp2 = BW
        new_U_KNOT = U_KNOT
      endif
c...  now refine in v
      if (vflag.eq.1) then
        write(*,*) "Refining v-dir..."
        call RefineKnotVectSolid_3D(new_MCP,P,new_U_KNOT,NCP,Q,V_KNOT,
     &    OCP,R,W_KNOT,temp2,x_knot_v,0,jcount,0,2,temp3,new_U_KNOT,
     &    new_V_KNOT, W_KNOT)
      else
        temp3 = temp2
        new_V_KNOT = V_KNOT
      endif
c...    now refine in w
      if (wflag.eq.1) then
        write(*,*) "Refining w-dir..."
        call RefineKnotVectSolid_3D(new_MCP,P,new_U_KNOT,new_NCP,Q,
     &    new_V_KNOT,OCP,R,W_KNOT,temp3,x_knot_w,0,0,kcount,3,
     &    new_BW,new_U_KNOT,new_V_KNOT,new_W_KNOT)
      else
        new_BW = temp3
        new_W_KNOT = W_KNOT
      endif
        deallocate(temp2)
        deallocate(temp3)
      
c...  Convert projective coordinates into physical coordinates
      new_B_NET = new_BW
      
      do i = 1,NSD
         write(*,*) "Redefine Bnet", i
         new_B_NET(:,:,:,i) = new_B_NET(:,:,:,i)/new_B_NET(:,:,:,NSD+1)
         
      enddo
           
c      ---------------------------------------------------------------

c
c     At this point we have a new control net and new knot vectors in each
c       direction. We must now create connectivities such that we may
c       transfer load and boundary conditions, periodicities, etc. from the
c       original mesh to the new one.


     

c     We need the connectivities from our original mesh in order to 
c      assign old data to the correct locations in the new mesh.
      call genIEN_INN_3D
      call gen_FACE_IEN_3D




c     Generate FACE_CON matrix connecting old faces to new ones. This is
c      structured so that FACE_CON(new_face_num) equals
c      the old face number.      
      call gen_FACE_CON_3D(FACE_CON,new_NFACE,new_MCP,new_NCP,new_OCP,
     &  uflag,vflag,wflag)



c     Generate EL_CON matrix connecting old elements to new ones. 
c       EL_CON(new_el_num) equals the old element number 
      call gen_EL_CON_3D(EL_CON, new_NEL, new_MCP, new_NCP, new_OCP,
     &  uflag,vflag,wflag)





c...  Create new IEN and INN arrays
        call gen_new_IEN_INN_3D(new_MCP,new_NCP,new_OCP,new_IEN,
     &       new_INN,new_NEL,new_NNODZ)




      
c...  Also create new face connectivity arrays
        call gen_new_FACE_IEN_3D(new_NFACE, new_NNODZ,
     &       new_FACE_IEN, new_FACE_OR,new_NEL,
     &       new_MCP, new_NCP,new_OCP,new_IEN)


c
c...  Create new IBC array assign face data
c
      call gen_FACE_DATA_3D(new_NNODZ,new_NFACE,FACE_CON,new_FACE_OR, 
     &     new_FACE_IEN, new_IPER,  new_IBC, new_IBC_FACE, new_LD_FACE,
     &     new_MCP,new_NCP,new_OCP, new_INN, new_B_NET,new_U_KNOT,
     &     new_V_KNOT, new_W_KNOT,new_DIR_BC)


c
c...  Assign element data
c
      call gen_EL_DATA_3D(new_NEL, EL_CON, new_LD_EL, new_EY, new_NU)





c
c...  Now we have assembled all of the new structures, we may replace
c       the old ones and write our results to refined_mesh.dat
c

c     commented out because the compiler is a real SOB.
c$$$c     Deallocate old structures and reallocate with new dimensions
c$$$      deallocate(U_KNOT)    
c$$$      deallocate(V_KNOT)
c$$$      deallocate(W_KNOT)
c$$$      deallocate(B_NET)
c$$$      deallocate(BW)
c$$$      deallocate(IPER)
c$$$      deallocate(IBC)
c$$$      deallocate(DIR_BC)
c$$$      deallocate(IBC_FACE)
c$$$      deallocate(LD_FACE)
c$$$      deallocate(LD_EL)
c$$$      deallocate(EY)
c$$$      deallocate(NU)
c$$$      deallocate(IEN) ! for the time being, IEN and INN don't need 
c$$$      deallocate(INN)       !  to be written as they are not part of mesh.dat

c...  assign common variables to new values
      NNODZ = new_NNODZ
      MCP = new_MCP
      NCP = new_NCP
      OCP = new_OCP
      NFACE = new_NFACE
      NEL = new_NEL



c$$$      allocate(U_KNOT(MCP+P+1))
c$$$      U_KNOT = new_U_KNOT
c$$$      allocate(V_KNOT(NCP+Q+1))
c$$$      V_KNOT = new_V_KNOT
c$$$      allocate(W_KNOT(OCP+R+1))
c$$$      W_KNOT = new_W_KNOT
c$$$      allocate(B_NET(MCP,NCP,OCP,NSD+1))
c$$$      B_NET = new_B_NET
c$$$      allocate(BW(MCP,NCP,OCP,NSD+1))
c$$$      BW = new_BW
c$$$      allocate(IPER(NNODZ))
c$$$      IPER = new_IPER
c$$$      allocate(IBC(NNODZ,NSD))
c$$$      IBC = new_IBC
c$$$      allocate(DIR_BC(NNODZ,NSD))
c$$$      DIR_BC = new_DIR_BC
c$$$      allocate(IBC_FACE(NFACE,NSD))
c$$$      IBC_FACE = new_IBC_FACE
c$$$      allocate(LD_FACE(NFACE,NSD))
c$$$      LD_FACE = new_LD_FACE
c$$$      allocate(LD_EL(NEL,NSD))
c$$$      LD_EL = new_LD_EL
c$$$      allocate(EY(NEL))
c$$$      EY = new_EY
c$$$      allocate(NU(NEL))
c$$$      NU = new_NU

     


c...  Write to file refined_mesh.dat
c      call output_3D ---- now obsolete
      write(*,*) "Outputing mesh..."
      call output_3D_2(new_U_KNOT,new_V_KNOT,new_W_KNOT,new_B_NET,
     &     new_IPER,new_IBC,new_DIR_BC,new_IBC_FACE,new_LD_FACE,
     &     new_LD_EL,new_EY,new_NU)

      stop

      end
