c
c       This subroutine generates the FACE_IEN matrix, which relates face 
c       numbers and local node numbers to the appropriate global node numbers. 
c       It will also generate FACE_OR which will contain the orientations
c       of a given face.
c
c       This version will identify all shape functions with support over 
c       an element on the boundary as associated with a particular face, 
c       even though some of these functions will not have support on the
c       face itself. This will allow for the calculation of gradients of
c       solutions up to the boundary should they be neccessary. This 
c       approach should be more robust than merely identifying shape 
c       functions on the face itself. For simplicity, the numbering will 
c	not be dependent on face orientation, as not doing so would 
c       introduce a host of different complications in the routines that
c       will utilize this connectivity array.
c
c       This is for the 3D code.
c
c       January 15, 2004
c
c       J. Austin Cottrell
c       CES Graduate Student
c       Institute for Computational Engineering Science
c       The University of Texas at Austin

 
	
	subroutine gen_FACE_IEN_3D

	use aAdjKeep		! this f90 module contains all allocatables
c       
	include "common.h"	! common file defines all variables
c       
c...    Local variables
	integer :: i,j,face,e

	 
	allocate(FACE_IEN(NFACE,(P+1)*(Q+1)*(R+1)))
	allocate(FACE_OR(NFACE))
c       
c...    Initialize matrices and variables
c       
	FACE_IEN = 0
	FACE_OR = 0
        face = 0    

	
c       This paragraph doesn't really make sense. FIXME/    
c...    Loop through faces assigning numbers and filling out matrices.
c          Face numbers will be assigned sequentially starting with the
c          u-v surface at w = 1, denoted (u,v,1). From there we count
c          the (u,1,w), then the (MCP,v,w), the (u,NCP,w),the (1,v,w),
c          and lastly (u,v,OCP) surface. Once the face has been labled,
c          we find the number of the element the face belongs to and
c          assign the local node numbers.
c
c
c       Face Orientation scheme: 
c           1 - surface (u,v,1)
c           2 - surface (u,1,w)
c           3 - surface (MCP,v,w)
c           4 - surface (u,NCP,w)
c           5 - surface (1,v,w)
c           6 - surface (u,v,OCP)



c...    Orientation 1
	e = 0
	if (CLOSED_W_flag.ne.1) then
	   do j = 1,NCP
	      do i = 1,MCP
		 if ((i .ge. (P+1)).and.(j .ge. (Q+1))) then
		    face = face + 1     ! face number
		    FACE_OR(face) = 1   ! face orientation
		    e = e + 1           ! element number
		    FACE_IEN(face,:) = IEN(e,:)
		 endif
	      enddo
	   enddo
	endif



c...    Orientation 2
	if (CLOSED_V_flag.ne.1) then
	   do j = R+1,OCP
	      e = (MCP-P)*(NCP-Q)*(j-(R+1))
	      do i = P+1,MCP
		 e = e + 1	! element number
		 face = face+1
		 FACE_OR(face) = 2
		 FACE_IEN(face,:) = IEN(e,:)
	      enddo
	   enddo
	endif



c...    Orientation 3
	if (CLOSED_U_flag.ne.1) then
	   do j = R+1,OCP
	      e = (MCP-P)*(NCP-Q)*(j-(R+1))
	      do i = Q+1,NCP
		 face = face+1
		 FACE_OR(face) = 3
		 e = e + (MCP-P)
		 FACE_IEN(face,:) = IEN(e,:)
	      enddo
	   enddo
	endif



c...    Orientation 4
	if (CLOSED_V_flag.ne.1) then
	   do j = R+1,OCP
	      e = (MCP-P)*(NCP-Q)*(j-R)+1
	      do i = P+1,MCP
		 face = face+1
		 FACE_OR(face) = 4
		 e = e - 1 
		 FACE_IEN(face,:) = IEN(e,:)
	      enddo
	   enddo
	endif



c...    Orientation 5
	if (CLOSED_U_flag.ne.1) then
	   do j = R+1,OCP
	      e = (MCP-P)*(NCP-Q)*(j-R)+1
	      do i = Q+1,NCP
		 face = face+1
		 FACE_OR(face) = 5
		 e = e - (MCP-P)
		 FACE_IEN(face,:) = IEN(e,:)
	      enddo
	   enddo
	endif


c...    Orientation 6
	if (CLOSED_W_flag.ne.1) then
	   e = (MCP-P)*(NCP-Q)*(OCP-R-1)
	   do j = 1,NCP
	      do i = 1,MCP
		 if ((i .ge. (P+1)).and.(j .ge. (Q+1))) then
		    face = face + 1
		    FACE_OR(face) = 6
		    e = e + 1
		    FACE_IEN(face,:) = IEN(e,:)
		 endif
	      enddo
	   enddo
	endif

	return
	end
