
      subroutine output_mp(U_KNOT,V_KNOT,W_KNOT,B_NET,IPER,IBC,
     &     DIR_BC,IBC_FACE,LD_FACE,LD_EL,EY,NU,meshflag,pflag,
     &     nknots,dir)

c------------------------------------------------------------------------c
c                                                                        c
c        Subroutine to open file and read vertex, edge                   c
c        and element numbers. Initialze parameters of interest.          c
c                                                                        c
c        3D Version                                                      c
c                                                                        c
c                                                                        c
c        Nov 15, 2003                                                    c
c                                                                        c
c        J. Austin Cottrell                                              c
c        CES Graduate Student                                            c
c        Institute for Computational Engineering Science                 c
c        The University of Texas at Austin                               c
c                                                                        c
c------------------------------------------------------------------------c

  

      include "common.h"

      integer i, j, k,l, meshf, workf, meshflag, pflag,
     &  nknots, maxtask, numtask, dir, count, count1, count2
      real*8 U_KNOT(MCP+P+1),V_KNOT(NCP+Q+1),W_KNOT(OCP+R+1),
     &     B_NET(MCP,NCP,OCP,NSD+1),DIR_BC(NNODZ,NSD),
     &     LD_FACE(NFACE,NSD), LD_EL(NEL,NSD),EY(NEL),NU(NEL)
      integer IPER(NNODZ),IBC(NNODZ,NSD), IBC_FACE(NFACE,NSD)
      character*30 fname
      character*10 cname
      logical even
      integer, allocatable :: itag(:),iacc(:),iother(:),
     &  numseg(:), isgbeg(:,:), lenseg(:,:)


      
      maxtask = 27
      
      allocate(itag(maxtask))
      allocate(iacc(maxtask))
      allocate(iother(maxtask))
      allocate(numseg(maxtask))
      
      if (dir.eq.1) then
        allocate(isgbeg(NCP*OCP,maxtask))
        allocate(lenseg(NCP*OCP,maxtask))
      elseif (dir.eq.3) then
        allocate(isgbeg(MCP*NCP,maxtask))
        allocate(lenseg(MCP*NCP,maxtask))
      endif
        
      meshf = 12
      workf = 13
      
      fname = trim('mesh' // cname(meshflag)) // '.dat'
      
c     
c...  Read in preliminary information
c
      open (meshf,file=fname, status='unknown')

      fname = trim('lwork' // cname(meshflag)) // '.dat'
      
      open (workf,file=fname, status='unknown')

      
      
      
      write (meshf,*)  NSD           ! number of spatial dimensions
      write (meshf,"(I2)", ADVANCE = "NO") P
      write (meshf,"(I2)", ADVANCE = "NO") Q
      write (meshf,"(I2)", ADVANCE = "YES") R ! degree of curves in u, v, and w direction
      write (meshf,"(I5)", ADVANCE = "NO") MCP
      write (meshf,"(I5)", ADVANCE = "NO") NCP
      write (meshf,"(I5)", ADVANCE = "YES") OCP ! number of control points in each direction



c
c...  Write knot vectors and control net
c
      do i = 1,MCP+P+1
         if (i < MCP+P+1) then
            write (meshf,"(F13.9)", ADVANCE = "NO") U_KNOT(i) ! write U_KNOT
         else
            write (meshf,"(F13.9)", ADVANCE = "YES") U_KNOT(i) 
         endif
      enddo

      do i = 1,NCP+Q+1
         if (i < NCP+Q+1) then
            write (meshf,"(F13.9)", ADVANCE = "NO") V_KNOT(i) ! write V_KNOT
         else
            write (meshf,"(F13.9)", ADVANCE = "YES") V_KNOT(i) 
         endif
      enddo

      do i = 1,OCP+R+1
         if (i < OCP+R+1) then
            write (meshf,"(F13.9)", ADVANCE = "NO") W_KNOT(i) ! write W_KNOT
         else
            write (meshf,"(F13.9)", ADVANCE = "YES") W_KNOT(i) 
         endif
      enddo

      do k = 1,(OCP)
         do j = 1,(NCP)
            do i = 1,(MCP)
               do l = 1,NSD+1
                  if (l < NSD+1) then
                    write (meshf,"(F16.9)", ADVANCE = "NO")
     &                    B_NET(i,j,k,l)
                  else
                    write (meshf,"(F16.9)", ADVANCE = "YES") 
     &                    B_NET(i,j,k,l)
                 endif
               enddo
           enddo
         enddo
      enddo
      
      !For Jessica's mesh
c      do k = 1,(OCP)
c         do j = 1,(NCP)
c            do i = 1,(MCP)
c               do l = 1,NSD+1
c                  if (l < NSD+1) then
c                     write (meshf,"(E20.12)", ADVANCE = "NO")
c     &                    B_NET(i,j,k,l)
c                  else
c                     write (meshf,"(E20.12)", ADVANCE = "YES") 
c     &                    B_NET(i,j,k,l)
c                  endif
c               enddo
c            enddo
c         enddo
c      enddo

      

   
c
c...  Write in Master/Slave relationships. These arise from periodic boundary
c       conditions, corners in the geometry, or any other case where two
c       control points must always remain equal.
c

      do i = 1, NNODZ
         write (meshf,*)  IPER(i) 
      enddo

c
c...  Write in Boundary/Face information
c
      write (meshf,"(I2)", ADVANCE = "NO") CLOSED_U_flag
      write (meshf,"(I2)", ADVANCE = "NO") CLOSED_V_flag
      write (meshf,"(I2)", ADVANCE = "YES") CLOSED_W_flag
                                ! flag determines if surface is closed
                                !  in some direction



c...  Write in boundary condition indicator at each node, in each direction
c     0 - nothing, 1 - Dir., 2 - Neu., 3 - Periodic
      do i = 1,NNODZ
         write (meshf,'(I2)', ADVANCE = "NO") IBC(i,1)
         write (meshf,'(I2)', ADVANCE = "NO") IBC(i,2)
         write (meshf,'(I2)', ADVANCE = "YES") IBC(i,3)
      enddo


c...  Write in Nodal displacements for inhomogeneous Dirichlet nodes.
      do i = 1, NNODZ
         write (meshf,"(F13.9)", ADVANCE = "NO") DIR_BC(i,1)
         write (meshf,"(F13.9)", ADVANCE = "NO") DIR_BC(i,2)
         write (meshf,"(F13.9)", ADVANCE = "YES") DIR_BC(i,3)
      enddo


c...  Write in boundary condition indicator on each face, in each direction
c     0 - nothing, 1 - constraint, 2 - load
      do i = 1,NFACE
         write (meshf,'(I2)', ADVANCE = "NO") IBC_FACE(i,1)
         write (meshf,'(I2)', ADVANCE = "NO") IBC_FACE(i,2)
         write (meshf,'(I2)', ADVANCE = "YES") IBC_FACE(i,3)
      enddo


c...  Write in actual load data on each face
c      This assumes (for the time being) that the load is contant
c      across each face. This will be modified in the future!
      do i = 1,NFACE
         write (meshf,"(F13.9)", ADVANCE = "NO") LD_FACE(i,1)
         write (meshf,"(F13.9)", ADVANCE = "NO") LD_FACE(i,2)
         write (meshf,"(F13.9)", ADVANCE = "YES") LD_FACE(i,3)
      enddo


     
c
c...  Write in body force loads on elements. This assumes constant
c       force across each element. We can modify this in the future.
      do i = 1,NEL
         write (meshf,"(F13.9)", ADVANCE = "NO") LD_EL(i,1)
         write (meshf,"(F13.9)", ADVANCE = "NO") LD_EL(i,2)
         write (meshf,"(F13.9)", ADVANCE = "YES") LD_EL(i,3)
      enddo

      
c
c...  Write in material data
c

c...  Young's Modulus in each element
      do i = 1,NEL
         write (meshf,*) EY(i)
      enddo

c... Poisson's ratio in each element
      do i = 1,NEL
         write (meshf,"(F13.9)") NU(i)
      enddo

c...  CREATE AND WRITE LOCAL WORK ARRAY...

      if (dir.eq.3) then        !decomposition in w 
        
        if (even(meshflag)) then ! Proc 2,4,6,...
          
          numtask = 2
          itag(1) = 100
          itag(2) = 100
          iacc(1) = 1
          iacc(2) = 0
          iother(1) = meshflag-1
          iother(2) = meshflag+1
          numseg(1) = 1
          numseg(2) = 1
          isgbeg(1,1) = 1
          isgbeg(1,2) = MCP*NCP*OCP-MCP*NCP+1
          lenseg(1,1) = MCP*NCP
          lenseg(1,2) = MCP*NCP
          
        else                    ! Procs 1,3,5,...
          
          numtask = 2
          itag(1) = 100
          itag(2) = 100
          iacc(1) = 0
          iacc(2) = 1
          iother(1) = meshflag+1
          iother(2) = meshflag-1
          numseg(1) = 1
          numseg(2) = 1
          isgbeg(1,1) = MCP*NCP*OCP-MCP*NCP+1
          isgbeg(1,2) = 1
          lenseg(1,1) = MCP*NCP
          lenseg(1,2) = MCP*NCP
          
        endif
        
                                ! Modify for left-most and right-most procs
        
        if (meshflag.eq.1) then
                                ! If first or last processor
          
          if(pflag.eq.0) then   ! If no periodic BCs
            numtask = 2
            iother(2) = -2
            iacc(2) = 1
          else                  ! Else
            numtask = 2
            iacc(2) = 0
            iother(2) = nknots+1
          endif
          
        elseif (meshflag.eq.(nknots+1)) then
          
          if(pflag.eq.0) then   ! If no periodic BCs
            numtask = 2
            iother(2) = -2
            iacc(2) = 1
          else                  ! Else
            numtask = 2
            iacc(2) = 1
            iother(2) = 1
          endif
          
        endif                   ! End Modify for left-most and right-most procs
        
        write(13,*) numtask, 13 ! 13 - length of the local work array for a 1-D decomposition
        
        do i = 1, numtask
          write(13,*) itag(i)
          write(13,*) iacc(i)
          write(13,*) iother(i)
          write(13,*) numseg(i)
          write(13,*) isgbeg(1,i)
          write(13,*) lenseg(1,i)
        enddo


        
      elseif (dir.eq.1) then    ! Dec. in U
        
c...    Special case when first n/2 procs touch last n/2 procs along v
        
c...    1 --> n
c...    2 --> n-1
c...    3 --> n-2          etc.

	
        if (meshflag.eq.1) then ! First Processor
          numtask = 2
          
          itag(1) = 100
          iacc(1) = 0
          iother(1) = meshflag+1
          numseg(1) = NCP*OCP-OCP
          count1 = 0
          count2 = 0
          do i = 1, OCP
            do j = 1, NCP
	    
              count1 = count1+1
              if (j.ne.1) then
                count2 = count2+1
                isgbeg(count2,1) = count1*MCP
                lenseg(count2,1) = 1
              endif
            enddo
          enddo
      
          itag(2) = 100
          iacc(2) = 0
          iother(2) = (nknots+1)+1-meshflag
          numseg(2) = MCP*OCP
          count1 = 0
          count2 = 0
          do i = 1, OCP
            do j = 1, NCP
	    
              count1 = count1+1
              count2 = count2+1
              isgbeg(count2,2) = j + (i-1)*MCP*NCP 
              lenseg(count2,2) = 1
            enddo
          enddo

        endif
        
        
        if ((meshflag.gt.1).and.(meshflag.le.(nknots+1)/2).and.
     &    (maxval(B_NET(:,1,1,1)).gt.1d+0)) then ! Top processors before Trailing Edge
       
          numtask = 4
          
          itag(1) = 100
          iacc(1) = 1
          iother(1) = meshflag-1
          numseg(1) = NCP*OCP-OCP
          count1 = 0
          count2 = 0
          do i = 1, OCP
            do j = 1, NCP
              count1 = count1+1
              if (j.ne.1) then
                count2 = count2+1
                isgbeg(count2,1) = count1*MCP-MCP+1
                lenseg(count2,1) = 1
              endif
            enddo
          enddo
          
          
          
          itag(2) = 100
          iacc(2) = 0
          iother(2) = meshflag+1
          numseg(2) = NCP*OCP-OCP
          count1 = 0
          count2 = 0
          do i = 1, OCP
            do j = 1, NCP
              count1 = count1+1
              if (j.ne.1) then
                count2 = count2+1
                isgbeg(count2,2) = count1*MCP
                lenseg(count2,2) = 1
              endif
            enddo
          enddo
          
          
          itag(3) = 100
          iacc(3) = 0
          iother(3) = (nknots+1)+1-meshflag
          numseg(3) = MCP*OCP-OCP
          count1 = 0
          count2 = 0
          do i = 1, OCP
            do j = 1, MCP
              count1 = count1+1
              if (j.ne.1) then
                count2 = count2+1
                isgbeg(count2,3) = j + (i-1)*MCP*NCP 
                lenseg(count2,3) = 1
              endif
            enddo
          enddo
          
          
          itag(4) = 100
          iacc(4) = 0
          iother(4) = (nknots+1)+1-meshflag+1
          numseg(4) = OCP
          do i = 1, OCP
            isgbeg(i,4) = 1+(i-1)*MCP*NCP
            lenseg(i,4) = 1
          enddo
          
          
        endif
        
        if ((meshflag.le.(nknots+1)/2).and.
     &    (maxval(B_NET(:,1,1,1)).eq.1d+0)) then ! Top first processor after Trailing Edge
           
          numtask = 3
          
          
          itag(1) = 100
          iacc(1) = 1
          iother(1) = meshflag-1
          numseg(1) = NCP*OCP-OCP
          count1 = 0
          count2 = 0
          do i = 1, OCP
            do j = 1, NCP
              count1 = count1+1
              if (j.ne.1) then
                count2 = count2+1
                isgbeg(count2,1) = count1*MCP-MCP+1
                lenseg(count2,1) = 1
              endif
            enddo
          enddo
          
          
          
          itag(2) = 100
          iacc(2) = 0
          iother(2) = meshflag+1
          numseg(2) = NCP*OCP
          count1 = 0
          count2 = 0
          do i = 1, OCP
            do j = 1, NCP
              count1 = count1+1
              count2 = count2+1
              isgbeg(count2,2) = count1*MCP
              lenseg(count2,2) = 1
            enddo
          enddo
          
          
          itag(3) = 100
          iacc(3) = 0
          iother(3) = (nknots+1)+1-meshflag+1
          numseg(3) = OCP
          do i = 1, OCP
            isgbeg(i,3) = 1+(i-1)*MCP*NCP
            lenseg(i,3) = 1
          enddo
          
          
        endif
        
        
        if (maxval(B_NET(:,1,1,1)).lt.1d+0) then ! Remaining Processors adjacent to Airfoil
           
          numtask = 2
          
          itag(1) = 100
          iacc(1) = 1
          iother(1) = meshflag-1
          numseg(1) = NCP*OCP
          do i = 1, numseg(1)
            isgbeg(i,1) = 1 + (i-1)*MCP
            lenseg(i,1) = 1
          enddo 
          
          itag(2) = 100
          iacc(2) = 0
          iother(2) = meshflag+1
          numseg(2) = NCP*OCP
          do i = 1, numseg(2)
            isgbeg(i,2) = i*MCP
            lenseg(i,2) = 1
          enddo
          
        endif
        
        if ((meshflag.ge.(nknots+1)/2).and.
     &    (maxval(B_NET(:,1,1,1)).eq.1d+0)) then ! Bottom last processor on Airfoil
    
          numtask = 2
          
          itag(1) = 100
          iacc(1) = 1
          iother(1) = meshflag-1
          numseg(1) = NCP*OCP
          do i = 1, numseg(1)
            isgbeg(i,1) = 1 + (i-1)*MCP
            lenseg(i,1) = 1
          enddo 
          
          itag(2) = 100
          iacc(2) = 0
          iother(2) = meshflag+1
          numseg(2) = NCP*OCP
          count1 = 0
          count2 = 0
          do i = 1, OCP
            do j = 1, NCP
              count1 = count1+1
              count2 = count2+1
              isgbeg(count2,2) = count1*MCP
              lenseg(count2,2) = 1
            enddo
          enddo
          
          
          
          
        endif
        
        if ((meshflag.lt.(nknots+1)).and.(meshflag.ge.(nknots+1)/2).and.
     &    (maxval(B_NET(:,1,1,1)).gt.1d+0)) then ! Bottom processors after Trailing Edge
          
          numtask = 4
                   
          
          itag(1) = 100
          iacc(1) = 1
          iother(1) = (nknots+1)+1-meshflag
          numseg(1) = MCP*OCP-OCP
          count1 = 0
          count2 = 0
          do i = 1, OCP
            do j = MCP,1,-1
              count1 = count1+1
              if (j.ne.MCP) then
                count2 = count2 + 1
                isgbeg(count2,1) = j + (i-1)*MCP*NCP 
                lenseg(count2,1) = 1
              endif
            enddo
          enddo
          
          
          itag(2) = 100
          iacc(2) = 1
          iother(2) = (nknots+1)+1-meshflag+1
          numseg(2) = OCP
          do i = 1, numseg(2)
            isgbeg(i,2) = 1+(i-1)*MCP*NCP
            lenseg(i,2) = 1
          enddo
          
          
          itag(3) = 100
          iacc(3) = 1
          iother(3) = meshflag-1
          numseg(3) = NCP*OCP
          count1 = 0
          count2 = 0
          do i = 1, OCP
            do j = 1, NCP
              count1 = count1+1
              count2 = count2+1
              isgbeg(count2,3) = count1*MCP-MCP+1
              lenseg(count2,3) = 1
            enddo
          enddo
          
          
          
          itag(4) = 100
          iacc(4) = 0
          iother(4) = meshflag+1
          numseg(4) = NCP*OCP
          count1 = 0
          count2 = 0
          do i = 1, OCP
            do j = 1, NCP
              count1 = count1+1
              count2 = count2+1
              isgbeg(count2,4) = count1*MCP
              lenseg(count2,4) = 1
            enddo
          enddo          
          
          
        endif
        
        
        if (meshflag.eq.(nknots+1)) then ! Last Processor
                   
          numtask = 3

          
          itag(1) = 100
          iacc(1) = 1
          iother(1) = (nknots+1)+1-meshflag
          numseg(1) = NCP*OCP
          count1 = 0
          count2 = 0
          do i = 1, OCP
            do j = NCP,1,-1
	              count1 = count1+1
              count2 = count2 + 1
              isgbeg(count2,1) = j + (i-1)*MCP*NCP 
              lenseg(count2,1) = 1
            enddo
          enddo
           
          itag(2) = 100
          iacc(2) = 1
          iother(2) = (nknots+1)+1-meshflag+1
          numseg(2) = OCP
          do i = 1, numseg(2)
            isgbeg(i,2) = 1+(i-1)*MCP*NCP
            lenseg(i,2) = 1
          enddo
                    
          itag(3) = 100
          iacc(3) = 1
          iother(3) = meshflag-1
          numseg(3) = NCP*OCP
          count1 = 0
          count2 = 0
          do i = 1, OCP
            do j = 1, NCP
	      count1 = count1+1
              count2 = count2+1
              isgbeg(count2,3) = count1*MCP-MCP+1
              lenseg(count2,3) = 1
            enddo
          enddo
          
          
        endif
        

        
        
        count = 1 +             ! numtask
     &    4 + 2*MCP*OCP +       ! Special case - v-master slave
     &    4 + 2*OCP +           ! w-edge
     &    4 + 2*NCP*OCP +       ! U-decomp
     &    4 + 2*NCP*OCP         
        
                                ! OVERESTIMATE
          
        write(13,*) numtask, count ! length of the local work array for a 1-D decomposition in u
        
        do i = 1, numtask
          write(13,*) itag(i)
          write(13,*) iacc(i)
          write(13,*) iother(i)
          write(13,*) numseg(i)
          do j = 1, numseg(i)
            write(13,*) isgbeg(j,i)
            write(13,*) lenseg(j,i)
          enddo
        enddo
        
        
        
      endif
      

      write(*,*)" Finished Patch Number: ", meshflag

 
      
      
      
      return
      end
      
      
      function cname (i)
      
      logical      beg
      character*10 cname,cc
      integer      il(0:8)
      
      integer i, ic0, ii, k
      
      ic0 = ICHAR("0")
      cc = " "
      ii = i
      
      il(0) = mod(ii,10)
      do k = 1,8
        ii = (ii - il(k-1)) / 10
        il(k) = mod (ii,10)
      enddo
      
      beg = .false.
      
      do k = 8,1,-1
        if (il(k) .ne. 0 .or. beg) then
          beg = .true.
          cc  = TRIM(cc) // CHAR(ic0 + il(k))
        endif
      enddo
      
      cc = TRIM(cc)//CHAR(ic0 + il(0))
      cname = "." // cc
      
      return
      end

      
      function even(i)
      integer i,j
      logical even
      j=i/2
      if(j*2 .eq. i)then
        even = .true.
      else
        even = .false.
      endif
      return
      end
