c
c  Subroutine e3LHS.f consumes the parameters lambda and mu, as well as the 
c     Gaussian quadrature weight and the spacial gradients, and assembles the
c     local stiffness matrix. 
c
c     This is the 3D version
c
c
c  January 20, 2004
c
c 
c  J. Austin Cottrell
c  Jurijs Bazilevs
c
c  CAM Graduate Students
c  Institute for Computational Engineering Science
c  The University of Texas at Austin


      subroutine e3LHS_3D (lambda, mu, gwt, shgradg, xKebe)

      use aAdjKeep

      include "common.h"


      integer aa, bb
      real*8  lambda, mu, gwt, shgradg(NSHL,NSD), 
     &     xKebe(NSD*NSD,NSHL,NSHL)


c...  loop over elements in each direction
      
      do aa = 1, NSHL
        
        do bb = 1, NSHL
            
          xKebe(1,aa,bb) = xKebe(1,aa,bb) + 
     &      ((lambda + 2d+0*mu)*shgradg(aa,1)*shgradg(bb,1) +
     &      mu*shgradg(aa,2)*shgradg(bb,2) +
     &      mu*shgradg(aa,3)*shgradg(bb,3))*DetJ*gwt
            
          xKebe(5,aa,bb) = xKebe(5,aa,bb) +
     &      (mu*shgradg(aa,1)*shgradg(bb,1) +
     &      (lambda + 2d+0*mu)*shgradg(aa,2)*shgradg(bb,2) +
     &      mu*shgradg(aa,3)*shgradg(bb,3))*DetJ*gwt
          
          xKebe(9,aa,bb) = xKebe(9,aa,bb) +
     &      (mu*shgradg(aa,1)*shgradg(bb,1) +
     &      mu*shgradg(aa,2)*shgradg(bb,2) +
     &      (lambda+2d+0*mu)*shgradg(aa,3)*shgradg(bb,3))*DetJ*gwt
          
          xKebe(2,aa,bb) = xKebe(2,aa,bb) + 
     &      (lambda*shgradg(aa,1)*shgradg(bb,2) +
     &      mu*shgradg(aa,2)*shgradg(bb,1))*DetJ*gwt
          
          xKebe(4,aa,bb) = xKebe(4,aa,bb) + 
     &      (mu*shgradg(aa,1)*shgradg(bb,2) +
     &      lambda*shgradg(aa,2)*shgradg(bb,1))*DetJ*gwt

          xKebe(3,aa,bb) = xKebe(3,aa,bb) + 
     &      (lambda*shgradg(aa,1)*shgradg(bb,3) +
     &      mu*shgradg(aa,3)*shgradg(bb,1))*DetJ*gwt
          
          xKebe(7,aa,bb) = xKebe(7,aa,bb) + 
     &      (mu*shgradg(aa,1)*shgradg(bb,3) +
     &      lambda*shgradg(aa,3)*shgradg(bb,1))*DetJ*gwt

          xKebe(6,aa,bb) = xKebe(6,aa,bb) + 
     &      (lambda*shgradg(aa,2)*shgradg(bb,3) +
     &      mu*shgradg(aa,3)*shgradg(bb,2))*DetJ*gwt
          
          xKebe(8,aa,bb) = xKebe(8,aa,bb) + 
     &      (mu*shgradg(aa,2)*shgradg(bb,3) +
     &      lambda*shgradg(aa,3)*shgradg(bb,2))*DetJ*gwt
          
        enddo
        
      enddo
      
      return
      end

      
      subroutine e3LHS_mass_3D (gwt, shg, xKebe)
      
      use aAdjKeep
      
      include "common.h"
      
      
      integer aa, bb
      real*8  gwt, shg(NSHL), 
     &  xKebe(NSD*NSD,NSHL,NSHL)
      

c...  loop over elements in each direction

c*      do aa = 1, NSHL
c*        
c*        do bb = 1, NSHL
c*          
c*          xKebe(1,aa,bb) = xKebe(1,aa,bb) + 
c*     &      shg(aa)*shg(bb)*DetJ*gwt
c*          
c*          xKebe(5,aa,bb) = xKebe(5,aa,bb) + 
c*     &      shg(aa)*shg(bb)*DetJ*gwt
c*          
c*          xKebe(9,aa,bb) = xKebe(9,aa,bb) + 
c*     &      shg(aa)*shg(bb)*DetJ*gwt
c*          
c*        enddo
c*        
c*      enddo


      do aa = 1, NSHL
        
        
        xKebe(1,aa,aa) = xKebe(1,aa,aa) + 
     &    shg(aa)*DetJ*gwt
        
        xKebe(5,aa,aa) = xKebe(5,aa,aa) + 
     &    shg(aa)*DetJ*gwt
        
        xKebe(9,aa,aa) = xKebe(9,aa,aa) + 
     &    shg(aa)*DetJ*gwt
        
        
      enddo
        
      return
      end
