c------------------------------------------------------------------------c
c                                                                        c
c        Main routine to call all the subroutines                        c
c                                                                        c
c                                                                        c
c------------------------------------------------------------------------c
      

  
      use aAdjKeep
      use common
      use mpi

      implicit none

c...  Global variables
      integer iunit, ierr
      integer
     &     iproc, numproc,
     &     Npx, Npy, Npz,
     &     istep, Nsteps,
     &     ini, fin, number

      integer, allocatable :: list(:,:), TimeSteps(:)
      integer pos, posn
      character*300 a
      character*30 fname
      character*10 cname

      character*10 inp;
c...  Initialize MPI
      call MPI_INIT(                                mpi_err )
      call MPI_COMM_SIZE( MPI_COMM_WORLD, numnodes, mpi_err )
      call MPI_COMM_RANK( MPI_COMM_WORLD, myid    , mpi_err )

c...  get main input
      if (myid.eq.mpi_master) write(*,*)
      !Number of processors in each direction (rep.dat)
      iunit = 101
      call getarg(1,inp); read(inp,*) Npx
      call getarg(2,inp); read(inp,*) Npy
      call getarg(3,inp); read(inp,*) Npz
c      Npx = 4
c      Npy = 1
c      Npz = 16
c      open (iunit, file="rep.dat",
c     &     form="formatted", status="old", iostat=ierr)
c      if (ierr == 0) then
c         read(iunit,*) Npx, Npy, Npz
c         close (iunit)
c         if (myid.eq.mpi_master) write(*,*)
c     &        "Number of Processors in each direction", Npx, Npy, Npz
c      else
c         call messenger_abort('Number of Processors in each direction?')
c      endif
      !Step Number (istart, nstep, inter)     (step.dat)
       Nsteps = 1
       allocate(TimeSteps(Nsteps))
       TimeSteps = 1
       write(*,*) "Number of steps requested", Nsteps
       write(*,*) "TimeSteps postprocessed", TimeSteps
c      open (iunit, file="step.dat",
c     &     form="formatted", status="old", iostat=ierr)
c      if (ierr == 0) then
c         read(iunit,*) Nsteps
c         allocate(TimeSteps(Nsteps))
c         read(iunit,*) TimeSteps
c         close (iunit)
c         if (myid.eq.mpi_master) then
c            write(*,*) "Number of steps requested", Nsteps
c            write(*,*) "TimeSteps postprocessed", TimeSteps
c         endif
c      else
c         call messenger_abort('Step.dat not found')
c      endif
      !Data for *case (ini, fin, number)     (list.dat)
      ini = 1
      fin = 1
      number = 1
      allocate(list(1,number))
      list(1,1) = 1
c      open (iunit, file="list.dat",
c     &     form="formatted", status="old", iostat=ierr)
c      if (ierr == 0) then
c         read(iunit,*) ini, fin, number
c         allocate(list(1,number))
c         read(iunit,*) list
c         close (iunit)
c         if (myid.eq.mpi_master) write(*,*)
c     &        "You will have the following steps", list(1,:)
c      else
c         call messenger_abort("list.dat not found")
c      endif
      !Read NDOF
c      fname = trim('restart/restart' //
c     &              cname(TimeSteps(1))) // cname (1)
c      open (45, file=fname, status='old')
c      read(45,*) 
c      read(45,'(A)') a
c      close(45)
c      NDOF=0
c      pos = 1
c      do 
c         posn = index(a(pos:), ".")
c         if(posn == 0) exit
c         NDOF = NDOF + 1
c         pos = pos + posn + 1
c      end do
      NDOF=1
      if (myid.eq.mpi_master) write(*,*)
     &     "The number of degrees of freedom is: ",NDOF
      if (myid.eq.mpi_master) write(*,*)
      if (myid.eq.mpi_master) write(*,*)

c...  Loop through timesteps
      numproc=Npx*Npy*Npz
      do istep = 1, Nsteps
         if (myid.eq.mpi_master) then
            write(*,*) 
            write(*,*) 'Time step',TimeSteps(istep)
         endif
         !For each processor
         do iproc = 1, numproc
            if (mod(iproc,numnodes).eq.myid)then
               !Read mesh data (from mesh.*.dat)
               call input_3D(iproc)
               !Read results from (restart.*.*)
               allocate(ug(NNODZ,NDOF))
               ug=0d0
               call gen_results(numproc)
               call results_3D(iproc,Timesteps(istep),numproc)
               NSHL = (P+1)*(Q+1)*(R+1)
               NGAUSS = 3       ! Visualization
               !generate IEN and  Coordinates
               call gen_IEN_INN_3D
               if (iproc.eq.1) then
                  write(*,"(a,i5)", ADVANCE = "NO") "  Patch No. ",iproc
               elseif (iproc.lt.numproc) then
                  write(*,"(i5)", ADVANCE = "NO") iproc
               else
                  write(*,"(i5)", ADVANCE = "yes") iproc
               endif
               !Generate sample points
               call gen_PTSU 
               !Geometry and write
               call VizGMV_3D(iproc,numproc,
     &              TimeSteps(istep),TimeSteps(Nsteps),
     &              ini, fin, number,list)
               !Deallocate arrays
               deallocate(ug)
               deallocate(U_KNOT)    
               deallocate(V_KNOT)
               deallocate(W_KNOT)
               deallocate(B_NET)
               deallocate(INN)
               deallocate(IEN)
               deallocate(PTSu)
               deallocate(PTSv)
               deallocate(PTSw)
               deallocate(uSOL)
               deallocate(xSOL)
            endif
         enddo
      enddo
c...  End loop through timesteps

      if (myid.eq.mpi_master) then
         write(*,*) 
         write(*,*) 'End'
      endif
c...  Finalice
      call MPI_Finalize(mpi_err)
      
      end



c######################################################################c
c######################################################################c
c###########################   Subroutines   ##########################c
c######################################################################c
c######################################################################c

!=======================================================================                                                                                                                                                                     

      subroutine messenger_abort(mess)
      use mpi
      implicit none
      integer ierr
      character*30 mess

      if (myid.eq.mpi_master) write(*,*)'Error:   ',mess
      ! Abort MPI                                                                                                                                                                                                  
      call MPI_abort (MPI_COMM_WORLD,ierr)

      return
      end subroutine messenger_abort



