!###########################################################################

      subroutine eval_SHAPE_3D_gui_2(
     &     e,
     &     u_hat,v_hat,w_hat,
     &     shl,shgradg)

      use aAdjKeep

      use common
      implicit none

c------------------------VARIABLE DECLARATIONS--------------------------------
c...  Element number
      integer e, ep, nders
c...  u and v coordinates of integration point in parent element
      real*8 u_hat,v_hat,w_hat,du, dv, dw, ds, ds2, ds3
c...  Vector of Local basis function values at (u_hat, v_hat), local and 
c          global gradients.
      real*8
     &     shl(NSHL),shgradl(NSHL,NSD),shgradg(NSHL,NSD),
     &     dxdxi(NSD,NSD), dxidx(NSD,NSD), 
     &     tempshl(NSHL),tempshgradl(NSHL,NSD)
c...  Local Variables
      !1D nonrational basis functions and derivs in u and v
      real*8 N(2,P+1), M(2,Q+1), O(2,R+1)
      !u and v coordinates of integration point, denominator and derivative sums
      real*8 u, v, w, denom_sum, derv_sum_U, derv_sum_V, derv_sum_W
      !NURBS coordinates, counters for loops
      integer ni, nj, nk, i, j, k, icount, aa, bb, cc
      !temporary variables
      real*8 tmp
c------------------------------------------------------------------

c...  Get NURBS coordinates for local node 1
      ni = INN(IEN(e,1),1)
      nj = INN(IEN(e,1),2)
      nk = INN(IEN(e,1),3)
c...  Get u and v coordinates of integration point
      u = u_hat
      v = v_hat
      w = w_hat

c...  Evaluate 1D shape functions and derivatives each direction
      call dersbasisfuns(ni,P,MCP,u,U_KNOT,N) !  u direction
      call dersbasisfuns(nj,Q,NCP,v,V_KNOT,M) !  v direction
      call dersbasisfuns(nk,R,OCP,w,W_KNOT,O) !  w direction
c...  Get knot span sizes
      du = U_KNOT(ni+1)-U_KNOT(ni)
      dv = V_KNOT(nj+1)-V_KNOT(nj)
      dw = W_KNOT(nk+1)-W_KNOT(nk)
c...  Form basis functions and derivatives dR/du and dR/dv      
      !Initialize
      icount = 0
      denom_sum = 0d0
      derv_sum_U = 0d0
      derv_sum_V = 0d0
      derv_sum_W = 0d0
      shgradl = 0d0
      tempshl = 0d0
      tempshgradl = 0d0
      !Loop through shape functions
      do k = 0, R
        do j = 0, Q
          do i = 0, P
            icount = icount+1
            !Basis functions
            shl(icount) = N(1,P+1-i)*M(1,Q+1-j)*O(1,R+1-k)*
     &        B_NET(ni-i,nj-j,nk-k,NSD+1)
            denom_sum = denom_sum + shl(icount)
            shgradl(icount,1) = 
     &           N(2,P+1-i)*M(1,Q+1-j)*O(1,R+1-k)*
     &           B_NET(ni-i,nj-j,nk-k,NSD+1) ! u
            derv_sum_U = derv_sum_U + shgradl(icount,1)
            shgradl(icount,2) = 
     &           N(1,P+1-i)*M(2,Q+1-j)*O(1,R+1-k)*
     &           B_NET(ni-i,nj-j,nk-k,NSD+1) ! v
            derv_sum_V = derv_sum_V + shgradl(icount,2)
            shgradl(icount,3) = 
     &           N(1,P+1-i)*M(1,Q+1-j)*O(2,R+1-k)*
     &           B_NET(ni-i,nj-j,nk-k,NSD+1) ! w
            derv_sum_W = derv_sum_W + shgradl(icount,3)
          enddo
        enddo
      enddo
      
c     Divide through by denominator
      
      tempshl = shl
      tempshgradl = shgradl
      
      ds  = 1d0/denom_sum
      ds2 = ds*ds
      !basis functions
      shl = shl*ds
      !derivatives 
      shgradl(:,1) = shgradl(:,1)*ds - (shl*derv_sum_U)*ds2
      shgradl(:,2) = shgradl(:,2)*ds - (shl*derv_sum_V)*ds2
      shgradl(:,3) = shgradl(:,3)*ds - (shl*derv_sum_W)*ds2

c...  Now calculate gradients 

      dxdxi = 0d0
      icount = 0
      do k = 0, R
         do j = 0, Q
            do i = 0, P
               icount = icount + 1
               dxdxi(1,1) = dxdxi(1,1) + B_NET(ni-i,nj-j,nk-k,1) *
     &              shgradl(icount,1)
               dxdxi(1,2) = dxdxi(1,2) + B_NET(ni-i,nj-j,nk-k,1) *
     &              shgradl(icount,2)
               dxdxi(1,3) = dxdxi(1,3) + B_NET(ni-i,nj-j,nk-k,1) *
     &              shgradl(icount,3)
               dxdxi(2,1) = dxdxi(2,1) + B_NET(ni-i,nj-j,nk-k,2) *
     &              shgradl(icount,1)
               dxdxi(2,2) = dxdxi(2,2) + B_NET(ni-i,nj-j,nk-k,2) *
     &              shgradl(icount,2)
               dxdxi(2,3) = dxdxi(2,3) + B_NET(ni-i,nj-j,nk-k,2) *
     &              shgradl(icount,3)
               dxdxi(3,1) = dxdxi(3,1) + B_NET(ni-i,nj-j,nk-k,3) *
     &              shgradl(icount,1)
               dxdxi(3,2) = dxdxi(3,2) + B_NET(ni-i,nj-j,nk-k,3) *
     &              shgradl(icount,2)
               dxdxi(3,3) = dxdxi(3,3) + B_NET(ni-i,nj-j,nk-k,3) *
     &              shgradl(icount,3)
            enddo
         enddo
      enddo
c.... compute the inverse of deformation gradient
      dxidx(1,1) =   dxdxi(2,2) * dxdxi(3,3)
     &     - dxdxi(3,2) * dxdxi(2,3)
      dxidx(1,2) =   dxdxi(3,2) * dxdxi(1,3)
     &     - dxdxi(1,2) * dxdxi(3,3)
      dxidx(1,3) =  dxdxi(1,2) * dxdxi(2,3)
     &     - dxdxi(1,3) * dxdxi(2,2)
      tmp = 1d0 /
     &     ( dxidx(1,1) * dxdxi(1,1)
     &     + dxidx(1,2) * dxdxi(2,1)
     &     + dxidx(1,3) * dxdxi(3,1) )
      dxidx(1,1) = dxidx(1,1) * tmp
      dxidx(1,2) = dxidx(1,2) * tmp
      dxidx(1,3) = dxidx(1,3) * tmp
      dxidx(2,1) = (dxdxi(2,3) * dxdxi(3,1)
     &     - dxdxi(2,1) * dxdxi(3,3)) * tmp
      dxidx(2,2) = (dxdxi(1,1) * dxdxi(3,3)
     &     - dxdxi(3,1) * dxdxi(1,3)) * tmp
      dxidx(2,3) = (dxdxi(2,1) * dxdxi(1,3)
     &     - dxdxi(1,1) * dxdxi(2,3)) * tmp
      dxidx(3,1) = (dxdxi(2,1) * dxdxi(3,2)
     &  - dxdxi(2,2) * dxdxi(3,1)) * tmp
      dxidx(3,2) = (dxdxi(3,1) * dxdxi(1,2)
     &     - dxdxi(1,1) * dxdxi(3,2)) * tmp
      dxidx(3,3) = (dxdxi(1,1) * dxdxi(2,2)
     &     - dxdxi(1,2) * dxdxi(2,1)) * tmp

c      DetJ = 1d0/tmp           ! Note that DetJ resides in common

      shgradg(:,1) =
     &     + shgradl(:,1) * dxidx(1,1)
     &     + shgradl(:,2) * dxidx(2,1)
     &     + shgradl(:,3) * dxidx(3,1)
      shgradg(:,2) =
     &     + shgradl(:,1) * dxidx(1,2)
     &     + shgradl(:,2) * dxidx(2,2)
     &     + shgradl(:,3) * dxidx(3,2)
      shgradg(:,3) =
     &     + shgradl(:,1) * dxidx(1,3)
     &     + shgradl(:,2) * dxidx(2,3)
     &     + shgradl(:,3) * dxidx(3,3)      


      return
      end
      
