#!/bin/bash 

error()
{
    RED='\033[1;31m'
    NC='\033[0m' # No Color
    # print an error and exit
    echo " "
    echo "Error:"
    echo -e " ${RED}$e${NC}"
    echo " "
    echo "Execution aborted"
    echo " "
    exit 1
}
warning()
{
    ORA='\033[0;33m'
    NC='\033[0m' # No Color
    # print an error and exit
    echo " "
    echo "Warning:"
    echo -e " ${ORA}$e${NC}"
    info="$info\n  ${ORA}$e${NC}"
}

clear 

#Get directories
   name_sim=$1
   dir_i=$PWD/automesh
   dir_a=$PWD/automesh/automesh
   dir_m=$PWD/$name_sim
   #dir_m=$PWD/automesh/automesh/Meshes
   #Checks
   if [ ! -d "$dir_i" ]; then
       e="Can't locate isomesh directory"
       error
   fi
   if [ ! -d "$dir_a" ]; then
       e="Can't locate automesh directory"
       error
   fi
   if [ ! -d "$dir_m" ]; then
       e="Can't locate Mesh directory"
       error
   fi
   
#Get parameters
	MIN=15
	MAX=32000
	rnd=$(($RANDOM % ($MAX + 1 - $MIN) + $MIN))
	geom=prostate      # File with coarse mesh defining geometry   
	geo_values=$rnd # Parameters for the geometry
	procx=$3      # Processor partitions in X direction
	procy=$4      # Processor partitions in Y direction
	procz=$5     # Processor partitions in Z direction
	elmx=$6    # Number of elements in X direction
	elmy=$7     # Number of elements in Y direction
	elmz=$8    # Number of elements in Z direction
	perx=$9     # Periodic BC Flag in X direction: 0= NO PERIODIC BC/ 1 = PERIODIC BC
	pery=${10}     # Periodic BC Flag in Y direction: 0= NO PERIODIC BC/ 1 = PERIODIC BC
	perz=${11}     # Periodic BC Flag in Z direction: 0= NO PERIODIC BC/ 1 = PERIODIC BC
	ordx=${12}     # Order in X direction
	ordy=${13}     # Order in Y direction
	ordz=${14}     # Order in Z direction
	insertu=${15}  # Knot file to be used for insertion in u dir (- = NONE)
	insertv=${16}  # Same for v
	insertw=${17}  # Same for w
	strx=${18}    # Ratio for elements (-1=>No ratio; <-1=>Asymmetric ratio; >0=>Symmetryc ratio)
	stry=${19}    # Ratio for elements (-1=>No ratio; <-1=>Asymmetric ratio; >0=>Symmetryc ratio)
	strz=${20}    # Ratio for elements (-1=>No ratio; <-1=>Asymmetric ratio; >0=>Symmetryc ratio)
	commu="cp"   # Communication files (c0==Generate lwork; cp==Generate comfiles; *==Don't generate)
	namenurbs=${21} #Name of nurbs file input
	
#Copy nurbs to directory correct (Create support file)
	cp $dir_m/$namenurbs $dir_i/gen_prostate/prostate.$rnd.dat
#Open, edit, and load the master
   echo "  Opening new file"
   #cp $dir_a/characteristics_backup.dat $dir_a/characteristics.dat
  # emacs $dir_a/characteristics.dat
   #source $dir_a/characteristics.dat
   geo_str=$(echo $geo_values | sed -e 's/ /_/g')

#Generate master.dat
   echo "geom="$geom".dat" > $dir_a/master.dat
   echo "insertu=\""$insertu"\"">> $dir_a/master.dat
   echo "insertv=\""$insertv"\"">> $dir_a/master.dat
   echo "insertw=\""$insertw"\"">> $dir_a/master.dat
   echo "per=\""$perx $pery $perz"\"" >> $dir_a/master.dat
   if [ "$geom" == "annulus" ]; then
       echo "elems=\""$(($elmx/4)) $elmy $elmz"\"" >> $dir_a/master.dat
       echo "procs=\""$(($procx/4)) $procy $procz"\"" >> $dir_a/master.dat
   else
       echo "elems=\""$elmx $elmy $elmz"\"" >> $dir_a/master.dat
       echo "procs=\""$procx $procy $procz"\"" >> $dir_a/master.dat
   fi
   echo "stretch=\""$strx $stry $strz"\"" >> $dir_a/master.dat
   echo "order=\""$ordx $ordy $ordz"\"" >> $dir_a/master.dat
   echo "commu=\""$commu"\"" >> $dir_a/master.dat

#Perform checkings
   if [ "$geom" == "annulus" ]; then
       if [ "$(( $procx % 4))" != "0" ] || [ "$(( $elmx % 4))" != "0" ]; then
	   e="Both the number elements and processors in x direction must be divisible by 4 in this mesh"
	   error
       fi
   fi

#Generate makemesh
   echo "#!/bin/bash" > $dir_a/makemesh
   echo "$dir_i/bin/gen_$geom $geo_values" >> $dir_a/makemesh
   echo "$dir_i/bin/meshgen" >> $dir_a/makemesh

#Generate folder name
   echo "Generating new folder"
   fname="G_"$geom"_"$geo_str"_E_"$elmx"_"$elmy"_"$elmz"_Proc_"$procx"_"$procy"_"$procz"_Or_"$ordx"_"$ordy"_"$ordz"_Per_"$perx"_"$pery"_"$perz"_C_"$commu"_S_"$strx"_"$stry"_"$strz"_R_"$insertu"_"$insertv"_"$insertw
   if [ -d $dir_m/$fname ]; then
       echo "  This mesh has already been generated"
	   echo "  Removing old mesh files"
	   rm -r $dir_m/$fname/*
   fi
   echo "  Folder name" $fname
   mkdir $dir_m/$fname 2> /dev/null
   #Copy master
   echo "  Copying files to folder"
   mv $dir_a/master.dat $dir_m/$fname/
   mv $dir_a/makemesh $dir_m/$fname/
   cd $dir_m/$fname

#If mesh is annulus
   if [ "$geom" == "annulus" ]; then
       if [ "$(( $procx % 4))" != "0" ] || [ "$(( $elmx % 4))" != "0" ]; then
	   e="Both the number elements and processors in x direction must be divisible by 4 in this mesh"
	   error
       fi
       mkdir temp
       for q in $(seq 1 4) ; do 
	   #Copy .dat and generate the mesh
	   if [ "$q" != "1" ]; then
	       echo "cp annulus.$q.dat annulus.dat" >> makemesh
	       echo "$dir_i/bin/meshgen" >> makemesh
	   fi
	   #Rename and save generated meshes
	   for j in $(seq 1 $procy); do 
	       for i in $(seq 1 $(($procx/4))) ; do 
		   n_local=$(( $procx/4*($j-1) +$i))
		   n_global=$(( $i + $procx/4*($q-1) + ($j-1)*$procx ))
		   echo "mv mesh.$n_local.dat temp/mesh.$n_global.dat" >> makemesh
	       done
	   done
       done
       #Use cube to get the communication files
       echo "sed -i 's/annulus/cube/' master.dat" >> makemesh
       echo "sed -i 's/elems=\""$(($elmx/4))"/elems=\""$elmx"/' master.dat" >> makemesh
       echo "sed -i 's/procs=\""$(($procx/4))"/procs=\""$procx"/' master.dat" >> makemesh
       echo "$dir_i/bin/gen_cube 0 1 0 1 0 0.1" >> makemesh
       echo "$dir_i/bin/meshgen" >> makemesh
       #Copy the annulus mesh files back
       echo "mv temp/mesh.*.dat ./" >> makemesh
       echo "rm -r temp" >> makemesh
   fi

#Generate the mesh
   echo "Generating the mesh"
   sh makemesh

   echo "  Mesh generated in folder: " $dir_m/$fname
   echo $dir_m/$fname > $dir_a/meshname.dat

#Delete support file created before
rm $dir_i/gen_prostate/prostate.$rnd.dat

cp $dir_i/postproc1/interfaz_postproc/a.out $dir_m/$fname/a.out
mkdir $dir_m/$fname/mesh
mkdir $dir_m/$fname/restart
cp $dir_m/$fname/mesh* $dir_m/$fname/mesh/
(cd $dir_m/$fname && ./a.out $procx $procy $procz)
rm -rf $dir_m/$fname/restart
rm -rf $dir_m/$fname/mesh
rm $dir_m/$fname/a.out
#cp $dir_m/$fname/mesh* $dir_i/postproc1/interfaz_postproc/mesh/
#(cd $dir_i/postproc1/interfaz_postproc && ./a.out $procx $procy $procz)
#mkdir $dir_m/$fname/meshprocessed
#cp $dir_i/postproc1/interfaz_postproc/var* $dir_m/$fname/meshprocessed/
#rm $dir_i/postproc1/interfaz_postproc/mesh/* 
#rm $dir_i/postproc1/interfaz_postproc/restart/* 
#rm $dir_i/postproc1/interfaz_postproc/var* 
